# -*- coding: utf-8 -*-

from opinel.utils.console import printError, printException, printInfo, printDebug

from ScoutSuite.providers.aws.services.awslambda import LambdaConfig
from ScoutSuite.providers.aws.services.cloudformation import CloudFormationConfig
from ScoutSuite.providers.aws.services.cloudtrail import CloudTrailConfig
from ScoutSuite.providers.aws.services.cloudwatch import CloudWatchConfig
from ScoutSuite.providers.aws.services.directconnect import DirectConnectConfig
from ScoutSuite.providers.aws.services.ec2 import EC2Config
from ScoutSuite.providers.aws.services.efs import EFSConfig
from ScoutSuite.providers.aws.services.elasticache import ElastiCacheConfig
from ScoutSuite.providers.aws.services.elb import ELBConfig
from ScoutSuite.providers.aws.services.elbv2 import ELBv2Config
from ScoutSuite.providers.aws.services.emr import EMRConfig
from ScoutSuite.providers.aws.services.iam import IAMConfig
from ScoutSuite.providers.aws.services.kms import KMSConfig
from ScoutSuite.providers.aws.services.rds import RDSConfig
from ScoutSuite.providers.aws.services.redshift import RedshiftConfig
from ScoutSuite.providers.aws.services.route53 import Route53Config, Route53DomainsConfig
from ScoutSuite.providers.aws.services.s3 import S3Config
from ScoutSuite.providers.aws.services.ses import SESConfig
from ScoutSuite.providers.aws.services.sns import SNSConfig
from ScoutSuite.providers.aws.services.sqs import SQSConfig
from ScoutSuite.providers.aws.services.vpc import VPCConfig
from ScoutSuite.providers.base.configs.services import BaseServicesConfig
from ScoutSuite.utils import format_service_name

try:
    from ScoutSuite.providers.aws.services.dynamodb_private import DynamoDBConfig
except ImportError:
    pass


class AWSServicesConfig(BaseServicesConfig):
    """
    Object that holds the necessary AWS configuration for all services in scope.
                                        
    :ivar cloudtrail:                   CloudTrail configuration
    :ivar cloudwatch:                   CloudWatch configuration: TODO
    :ivar dynamodb:                     DynomaDB configuration
    :ivar ec2:                          EC2 configuration
    :ivar iam:                          IAM configuration
    :ivar kms:                          KMS configuration
    :ivar rds:                          RDS configuration
    :ivar redshift:                     Redshift configuration
    :ivar s3:                           S3 configuration
    :ivar ses:                          SES configuration: TODO
    "ivar sns:                          SNS configuration
    :ivar sqs:                          SQS configuration
    """

    def __init__(self, metadata=None, thread_config=4, **kwargs):

        self.cloudformation = CloudFormationConfig(metadata['management']['cloudformation'], thread_config)
        self.cloudtrail = CloudTrailConfig(metadata['management']['cloudtrail'], thread_config)
        self.cloudwatch = CloudWatchConfig(metadata['management']['cloudwatch'], thread_config)
        self.directconnect = DirectConnectConfig(metadata['network']['directconnect'], thread_config)
        self.ec2 = EC2Config(metadata['compute']['ec2'], thread_config)
        self.efs = EFSConfig(metadata['storage']['efs'], thread_config)
        self.elasticache = ElastiCacheConfig(metadata['database']['elasticache'], thread_config)
        self.elb = ELBConfig(metadata['compute']['elb'], thread_config)
        self.elbv2 = ELBv2Config(metadata['compute']['elbv2'], thread_config)
        self.emr = EMRConfig(metadata['analytics']['emr'], thread_config)
        self.iam = IAMConfig(thread_config)
        self.kms = KMSConfig(metadata['security']['kms'], thread_config)
        self.awslambda = LambdaConfig(metadata['compute']['awslambda'], thread_config)
        self.redshift = RedshiftConfig(metadata['database']['redshift'], thread_config)
        self.rds = RDSConfig(metadata['database']['rds'], thread_config)
        self.route53 = Route53Config(thread_config)
        self.route53domains = Route53DomainsConfig(thread_config)
        self.s3 = S3Config(thread_config)
        self.ses = SESConfig(metadata['messaging']['ses'], thread_config)
        self.sns = SNSConfig(metadata['messaging']['sns'], thread_config)
        self.sqs = SQSConfig(metadata['messaging']['sqs'], thread_config)
        self.vpc = VPCConfig(metadata['network']['vpc'], thread_config)

        try:
            self.dynamodb = DynamoDBConfig(metadata['database']['dynamodb'], thread_config)
        except NameError as e:
            pass

    def _is_provider(self, provider_name):
        return provider_name == 'aws'

    # TODO is this ever called?
    # def single_service_pass(self):
    #     pass

    # TODO is this ever called?
    # def multi_service_pass(self):
    #     pass

    # TODO is this ever called?
    # def postprocessing(self):
    #     for service in self.services:
    #         method_name = '%s_postprocessing' % service
    #         if method_name in globals():
    #             try:
    #                 printInfo('Post-processing %s config...' % format_service_name(service))
    #                 method = globals()[method_name]
    #                 method(aws_config)
    #             except Exception as e:
    #                 printException(e)
    #                 pass



# -*- coding: utf-8 -*-

from botocore.exceptions import ClientError
from opinel.utils.aws import connect_service, handle_truncated_response
from opinel.utils.console import printError, printException
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.base import AWSBaseConfig
from ScoutSuite.utils import *


class IAMConfig(AWSBaseConfig):
    """
    Object that holds the IAM configuration

    :ivar credential_reports:    Credential report as downloaded from the AWS API
    :ivar groups:               Dictionary of IAM groups in the AWS account
    :ivar groups_count:         len(groups)
    :ivar password_policy:      Account password policy
    :ivar permissions:          Summary of permissions granted via all IAM policies
    :ivar policies:             Dictionary of IAM managed policies in use within the AWS Account
    :ivar policies_count:       len(policies)
    :ivar roles:                Dictionary of IAM roles in the AWS account
    :ivar roles_count:          len(roles)
    :ivar users:                Dictionary of IAM users in the AWS account
    :ivar users_count:          len(users)
    """

    targets = (

        ('groups', 'Groups', 'list_groups', {}, False),
        ('policies', 'Policies', 'list_policies', [{'Scope': 'Local'}, {'OnlyAttached': True}], False),
        ('roles', 'Roles', 'list_roles', {}, False),
        ('users', 'Users', 'list_users', {}, False),
        ('credential_reports', '', '', {}, False),
        ('password_policy', '', '', {}, False)
        # TODO: Federations
    )

    def __init__(self, target_config):
        self.credential_reports = {}
        self.groups = {}
        self.password_policy = {}
        self.permissions = {}
        self.policies = {}
        self.roles = {}
        self.users = {}
        super(IAMConfig, self).__init__(target_config)

    ########################################
    ##### Overload to fetch credentials report before and after
    ########################################

    def fetch_all(self, credentials, regions=None, partition_name='aws', targets=None):
        regions = [] if regions is None else regions
        self.fetch_credential_reports(credentials, True)
        super(IAMConfig, self).fetch_all(credentials, regions, partition_name, targets)
        self.fetch_password_policy(credentials)
        self.fetch_credential_reports(credentials)
        self.fetchstatuslogger.show(True)

    ########################################
    ##### Credential report
    ########################################

    def fetch_credential_reports(self, credentials, ignore_exception=False):
        """
        Fetch the credential report

        :param: api_client
        :type: FOO
        :param: ignore_exception : initiate credential report creation as not  always ready
        :type: Boolean
        """
        credential_reports = {}
        try:
            api_client = connect_service('iam', credentials, silent=True)
            response = api_client.generate_credential_report()
            if response['State'] != 'COMPLETE':
                if not ignore_exception:
                    printError('Failed to generate a credential report.')
                return

            report = api_client.get_credential_report()['Content']
            lines = report.splitlines()
            keys = lines[0].decode('utf-8').split(',')
            self.fetchstatuslogger.counts['credential_reports']['discovered'] = len(lines) - 1

            for line in lines[1:]:
                credential_report = {}
                values = line.decode('utf-8').split(',')
                user_id = values[0]
                for key, value in zip(keys, values):
                    credential_report[key] = value

                credential_report['password_last_used'] = self._sanitize_date(credential_report['password_last_used'])
                credential_report['access_key_1_last_used_date'] = self._sanitize_date(
                    credential_report['access_key_1_last_used_date'])
                credential_report['access_key_2_last_used_date'] = self._sanitize_date(
                    credential_report['access_key_2_last_used_date'])
                credential_report['last_used'] = self._compute_last_used(credential_report)
                credential_report['name'] = user_id
                credential_report['id'] = user_id
                manage_dictionary(credential_reports, user_id, credential_report)
                self.fetchstatuslogger.counts['credential_reports']['fetched'] = len(credential_reports)

            self.credential_reports = credential_reports

        except Exception as e:
            if ignore_exception:
                return
            printError('Failed to download a credential report.')
            printException(e)

    def _compute_last_used(self, credential_report):
        dates = [credential_report['password_last_used'],
                 credential_report['access_key_1_last_used_date'],
                 credential_report['access_key_2_last_used_date']]

        dates = [date for date in dates if date is not None]
        return max(dates) if len(dates) > 0 else None

    def _sanitize_date(self, date):
        """
        Returns the date if it is not equal to 'N/A' or 'no_information', else returns None
        """
        return date if date != 'no_information' and date != 'N/A' else None

    ########################################
    ##### Groups
    ########################################

    def parse_groups(self, group, params):
        """
        Parse a single IAM group and fetch additional information
        """
        # When resuming upon throttling error, skip if already fetched
        if group['GroupName'] in self.groups:
            return
        api_client = params['api_client']
        # Ensure consistent attribute names across resource types
        group['id'] = group.pop('GroupId')
        group['name'] = group.pop('GroupName')
        group['arn'] = group.pop('Arn')
        # Get group's members
        group['users'] = self.__fetch_group_users(api_client, group['name']);
        # Get inline policies
        policies = self.__get_inline_policies(api_client, 'group', group['id'], group['name'])
        if len(policies):
            group['inline_policies'] = policies
        group['inline_policies_count'] = len(policies)
        self.groups[group['id']] = group

    ########################################
    ##### Managed policies
    ########################################

    def parse_policies(self, fetched_policy, params):
        """
        Parse a single IAM policy and fetch additional information
        """
        api_client = params['api_client']
        policy = {}
        policy['name'] = fetched_policy.pop('PolicyName')
        policy['id'] = fetched_policy.pop('PolicyId')
        policy['arn'] = fetched_policy.pop('Arn')
        # Download version and document
        policy_version = api_client.get_policy_version(PolicyArn=policy['arn'],
                                                       VersionId=fetched_policy['DefaultVersionId'])
        policy_version = policy_version['PolicyVersion']
        policy['PolicyDocument'] = policy_version['Document']
        # Get attached IAM entities
        policy['attached_to'] = {}
        attached_entities = handle_truncated_response(api_client.list_entities_for_policy, {'PolicyArn': policy['arn']},
                                                      ['PolicyGroups', 'PolicyRoles', 'PolicyUsers'])
        for entity_type in attached_entities:
            resource_type = entity_type.replace('Policy', '').lower()
            if len(attached_entities[entity_type]):
                policy['attached_to'][resource_type] = []
            for entity in attached_entities[entity_type]:
                name_field = entity_type.replace('Policy', '')[:-1] + 'Name'
                resource_name = entity[name_field]
                policy['attached_to'][resource_type].append({'name': resource_name})
        # Save policy
        self.policies[policy['id']] = policy

    ########################################
    ##### Password policy
    ########################################

    def fetch_password_policy(self, credentials):
        """
        Fetch the password policy that applies to all IAM users within the AWS account
        """
        self.fetchstatuslogger.counts['password_policy']['discovered'] = 0
        self.fetchstatuslogger.counts['password_policy']['fetched'] = 0
        try:
            api_client = connect_service('iam', credentials, silent=True)
            self.password_policy = api_client.get_account_password_policy()['PasswordPolicy']
            if 'PasswordReusePrevention' not in self.password_policy:
                self.password_policy['PasswordReusePrevention'] = False
            else:
                self.password_policy['PreviousPasswordPrevented'] = self.password_policy['PasswordReusePrevention']
                self.password_policy['PasswordReusePrevention'] = True
            # There is a bug in the API: ExpirePasswords always returns false
            if 'MaxPasswordAge' in self.password_policy:
                self.password_policy['ExpirePasswords'] = True
            self.fetchstatuslogger.counts['password_policy']['discovered'] = 1
            self.fetchstatuslogger.counts['password_policy']['fetched'] = 1

        except ClientError as e:
            if e.response['Error']['Code'] == 'NoSuchEntity':
                self.password_policy = {}
                self.password_policy[
                    'MinimumPasswordLength'] = '1'  # As of 10/10/2016, 1-character passwords were authorized when no policy exists, even though the console displays 6
                self.password_policy['RequireUppercaseCharacters'] = False
                self.password_policy['RequireLowercaseCharacters'] = False
                self.password_policy['RequireNumbers'] = False
                self.password_policy['RequireSymbols'] = False
                self.password_policy['PasswordReusePrevention'] = False
                self.password_policy['ExpirePasswords'] = False
            else:
                raise e
        except Exception as e:
            printError(str(e))

    ########################################
    ##### Roles
    ########################################

    def parse_roles(self, fetched_role, params):
        """
        Parse a single IAM role and fetch additional data
        """
        role = {}
        role['instances_count'] = 'N/A'
        # When resuming upon throttling error, skip if already fetched
        if fetched_role['RoleName'] in self.roles:
            return
        api_client = params['api_client']
        # Ensure consistent attribute names across resource types
        role['id'] = fetched_role.pop('RoleId')
        role['name'] = fetched_role.pop('RoleName')
        role['arn'] = fetched_role.pop('Arn')
        # Get other attributes
        get_keys(fetched_role, role, ['CreateDate', 'Path'])
        # Get role policies
        policies = self.__get_inline_policies(api_client, 'role', role['id'], role['name'])
        if len(policies):
            role['inline_policies'] = policies
        role['inline_policies_count'] = len(policies)
        # Get instance profiles
        profiles = handle_truncated_response(api_client.list_instance_profiles_for_role, {'RoleName': role['name']},
                                             ['InstanceProfiles'])
        manage_dictionary(role, 'instance_profiles', {})
        for profile in profiles['InstanceProfiles']:
            manage_dictionary(role['instance_profiles'], profile['InstanceProfileId'], {})
            role['instance_profiles'][profile['InstanceProfileId']]['arn'] = profile['Arn']
            role['instance_profiles'][profile['InstanceProfileId']]['name'] = profile['InstanceProfileName']
        # Get trust relationship
        role['assume_role_policy'] = {}
        role['assume_role_policy']['PolicyDocument'] = fetched_role.pop('AssumeRolePolicyDocument')
        # Save role
        self.roles[role['id']] = role

    ########################################
    ##### Users
    ########################################

    def parse_users(self, user, params):
        """
        Parse a single IAM user and fetch additional data
        """
        if user['UserName'] in self.users:
            return
        api_client = params['api_client']
        # Ensure consistent attribute names across resource types
        user['id'] = user.pop('UserId')
        user['name'] = user.pop('UserName')
        user['arn'] = user.pop('Arn')
        policies = self.__get_inline_policies(api_client, 'user', user['id'], user['name'])
        if len(policies):
            user['inline_policies'] = policies
        user['inline_policies_count'] = len(policies)
        user['groups'] = []
        groups = handle_truncated_response(api_client.list_groups_for_user, {'UserName': user['name']}, ['Groups'])[
            'Groups']
        for group in groups:
            user['groups'].append(group['GroupName'])
        try:
            user['LoginProfile'] = api_client.get_login_profile(UserName=user['name'])['LoginProfile']
        except Exception as e:
            pass
        user['AccessKeys'] = api_client.list_access_keys(UserName=user['name'])['AccessKeyMetadata']
        user['MFADevices'] = api_client.list_mfa_devices(UserName=user['name'])['MFADevices']
        # TODO: Users signing certss
        self.users[user['id']] = user

    ########################################
    ##### Finalize IAM config
    ########################################

    def finalize(self):
        # Update permissions for managed policies
        for policy_id in self.policies:
            if 'attached_to' in self.policies[policy_id] and len(self.policies[policy_id]['attached_to']) > 0:
                for entity_type in self.policies[policy_id]['attached_to']:
                    for entity in self.policies[policy_id]['attached_to'][entity_type]:
                        entity['id'] = self.get_id_for_resource(entity_type, entity['name'])
                        entities = getattr(self, entity_type)
                        manage_dictionary(entities[entity['id']], 'policies', [])
                        manage_dictionary(entities[entity['id']], 'policies_counts', 0)
                        entities[entity['id']]['policies'].append(policy_id)
                        entities[entity['id']]['policies_counts'] += 1
                        self.__parse_permissions(policy_id, self.policies[policy_id]['PolicyDocument'], 'policies',
                                                 entity_type, entity['id'])
            else:
                self.__parse_permissions(policy_id, self.policies[policy_id]['PolicyDocument'], 'policies', None, None)
        super(IAMConfig, self).finalize()

    ########################################
    ##### Class helpers
    ########################################

    def get_id_for_resource(self, iam_resource_type, resource_name):
        for resource_id in getattr(self, iam_resource_type):
            if getattr(self, iam_resource_type)[resource_id]['name'] == resource_name:
                return resource_id

    def __fetch_group_users(self, api_client, group_name):
        users = []
        fetched_users = api_client.get_group(GroupName=group_name)['Users']
        for user in fetched_users:
            users.append(user['UserId'])
        return users

    ########################################
    ##### Inline policies
    ########################################

    def __get_inline_policies(self, api_client, iam_resource_type, resource_id, resource_name):
        fetched_policies = {}
        get_policy_method = getattr(api_client, 'get_' + iam_resource_type + '_policy')
        list_policy_method = getattr(api_client, 'list_' + iam_resource_type + '_policies')
        args = {}
        args[iam_resource_type.title() + 'Name'] = resource_name
        try:
            policy_names = list_policy_method(**args)['PolicyNames']
        except Exception as e:
            if is_throttled(e):
                raise e
            else:
                printException(e)
                return fetched_policies
        try:
            for policy_name in policy_names:
                args['PolicyName'] = policy_name
                policy_document = get_policy_method(**args)['PolicyDocument']
                policy_id = self.get_non_provider_id(policy_name)
                manage_dictionary(fetched_policies, policy_id, {})
                fetched_policies[policy_id]['PolicyDocument'] = policy_document
                fetched_policies[policy_id]['name'] = policy_name
                self.__parse_permissions(policy_id, policy_document, 'inline_policies', iam_resource_type + 's',
                                         resource_id)
        except Exception as e:
            if is_throttled(e):
                raise e
            else:
                printException(e)
        return fetched_policies

    def __parse_permissions(self, policy_name, policy_document, policy_type, iam_resource_type, resource_name):
        # Enforce list of statements (Github issue #99)
        if type(policy_document['Statement']) != list:
            policy_document['Statement'] = [policy_document['Statement']]
        for statement in policy_document['Statement']:
            self.__parse_statement(policy_name, statement, policy_type, iam_resource_type, resource_name)

    def __parse_statement(self, policy_name, statement, policy_type, iam_resource_type, resource_name):
        # Effect
        effect = str(statement['Effect'])
        # Action or NotAction
        action_string = 'Action' if 'Action' in statement else 'NotAction'
        if type(statement[action_string]) != list:
            statement[action_string] = [statement[action_string]]
        # Resource or NotResource
        resource_string = 'Resource' if 'Resource' in statement else 'NotResource'
        if type(statement[resource_string]) != list:
            statement[resource_string] = [statement[resource_string]]
        # Condition
        condition = statement['Condition'] if 'Condition' in statement else None
        manage_dictionary(self.permissions, action_string, {})
        if iam_resource_type == None:
            return
        self.__parse_actions(effect, action_string, statement[action_string], resource_string,
                             statement[resource_string], iam_resource_type, resource_name, policy_name, policy_type,
                             condition)

    def __parse_actions(self, effect, action_string, actions, resource_string, resources, iam_resource_type,
                        iam_resource_name, policy_name, policy_type, condition):
        for action in actions:
            manage_dictionary(self.permissions[action_string], action, {})
            manage_dictionary(self.permissions[action_string][action], iam_resource_type, {})
            manage_dictionary(self.permissions[action_string][action][iam_resource_type], effect, {})
            manage_dictionary(self.permissions[action_string][action][iam_resource_type][effect], iam_resource_name, {})
            self.__parse_action(effect, action_string, action, resource_string, resources, iam_resource_type,
                                iam_resource_name, policy_name, policy_type, condition)

    def __parse_action(self, effect, action_string, action, resource_string, resources, iam_resource_type,
                       iam_resource_name, policy_name, policy_type, condition):
        for resource in resources:
            self.__parse_resource(effect, action_string, action, resource_string, resource, iam_resource_type,
                                  iam_resource_name, policy_name, policy_type, condition)

    def __parse_resource(self, effect, action_string, action, resource_string, resource, iam_resource_type,
                         iam_resource_name, policy_name, policy_type, condition):
        manage_dictionary(self.permissions[action_string][action][iam_resource_type][effect][iam_resource_name],
                          resource_string, {})
        manage_dictionary(
            self.permissions[action_string][action][iam_resource_type][effect][iam_resource_name][resource_string],
            resource, {})
        manage_dictionary(
            self.permissions[action_string][action][iam_resource_type][effect][iam_resource_name][resource_string][
                resource], policy_type, {})
        manage_dictionary(
            self.permissions[action_string][action][iam_resource_type][effect][iam_resource_name][resource_string][
                resource][policy_type], policy_name, {})
        self.permissions[action_string][action][iam_resource_type][effect][iam_resource_name][resource_string][
            resource][policy_type][policy_name]['condition'] = condition


# -*- coding: utf-8 -*-

from opinel.utils.console import printError

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig


class StackdriverMonitoringConfig(GCPBaseConfig):
    targets = (
        # ('resources', 'Resources', 'list_monitored_resource_descriptors', {'name': 'project_placeholder'}, False),
    )

    def __init__(self, thread_config):
        self.library_type = 'cloud_client_library'

        self.resources = {}
        self.resources_count = 0

        super(StackdriverMonitoringConfig, self).__init__(thread_config)

    def parse_resources(self, resource, params):
        a = 1



# -*- coding: utf-8 -*-

# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

import itertools
import json
import re

from google.api_core.exceptions import PermissionDenied
from google.cloud import container_v1
from googleapiclient.errors import HttpError

from opinel.utils.console import printException, printError

from ScoutSuite.providers.base.configs.base import BaseConfig
from ScoutSuite.providers.gcp.utils import gcp_connect_service

class GCPBaseConfig(BaseConfig):

    def __init__(self, thread_config=4, projects=None, **kwargs):
        projects = [] if projects is None else projects

        self.projects = projects

        self.error_list = []  # list of errors, so that we don't print the same error multiple times

        self.zones = None
        self.regions = None

        super(GCPBaseConfig, self).__init__(thread_config)

    def _is_provider(self, provider_name):
        return provider_name == 'gcp'

    def get_regions(self, **kwargs):
        """
        Certain services require to be poled per-region.
        In these cases, this method will return a list of regions to poll.

        :return:
        """

        computeengine_client = gcp_connect_service(service='computeengine')

        if not self.regions:
            # get regions from a project that has CE API enabled
            for project in self.projects:
                try:
                    regions_list = []
                    regions = computeengine_client.regions().list(project=project['projectId']).execute()['items']
                    for region in regions:
                        regions_list.append(region['name'])
                    self.regions = regions_list
                except HttpError as e:
                    pass
                except Exception as e:
                    printException(e)
                if self.regions:
                    break

        return self.regions

    def get_zones(self, **kwargs):
        """
        Certain services require to be poled per-zone.
        In these cases, this method will return a list of zones to poll.

        :return:
        """

        computeengine_client = gcp_connect_service(service='computeengine')

        if not self.zones:
            # get zones from a project that has CE API enabled
            for project in self.projects:
                try:
                    zones_list = []
                    zones = computeengine_client.zones().list(project=project['projectId']).execute()['items']
                    for zone in zones:
                        zones_list.append(zone['name'])
                    self.zones = zones_list
                except HttpError as e:
                    pass
                except Exception as e:
                    printException(e)
                if self.zones:
                    break

        return self.zones

    def _get_method(self, api_client, target_type, list_method_name):
        """
        Gets the appropriate method, required as each provider may have particularities

        :return:
        """

        # This is a specific case for GCP services that don't have a native cloud library
        if self.library_type == 'api_client_library':
            # Required for nested targers, e.g. projects().resource().list()
            target = getattr(api_client, target_type.split('.')[0])
            for t in target_type.split('.')[1:]:
                target = getattr(target(), t)
                # target_type = type  # hack to display the last element
            method = getattr(target(), list_method_name)
        # This works for AWS and GCP cloud libraries
        else:
            method = getattr(api_client, list_method_name)

        return method

    def _get_targets(self, response_attribute, api_client, method, list_params, ignore_list_error):
        """
        Fetch the targets, required as each provider may have particularities

        :return:
        """

        targets = []

        try:

            regions = self.get_regions()
            zones = self.get_zones()

            # Create a list with all combinations for method parameters
            list_params_list = []

            # Dict for all the elements to combine
            combination_elements = {'project_placeholder': [project['projectId'] for project in self.projects],
                                    'region_placeholder': regions,
                                    'zone_placeholder': zones}

            # Get a list of {{}} terms
            sources = re.findall("{{(.*?)}}", str(list_params.values()))
            # Remove keys from combinations if they aren't in the sources
            confirmed_combination_elements = {}
            for source in sources:
                confirmed_combination_elements[source] = combination_elements[source]
            # Build a list of the possible combinations
            combinations = self._dict_product(confirmed_combination_elements)
            for combination in combinations:
                l = list_params.copy()
                for k, v in l.items():
                    k1 = re.findall("{{(.*?)}}", v)
                    if k1:
                        l[k] = l[k].replace('{{%s}}' % k1[0], combination[k1[0]])
                list_params_list.append(l)

            for list_params_combination in list_params_list:

                try:

                    if self.library_type == 'cloud_client_library':

                        # TODO this should be more modular
                        # this is only for stackdriverlogging
                        if self.service == 'stackdriverlogging':
                            api_client.project = list_params_combination.pop('project')

                        response = method(**list_params_combination)

                        # TODO this should be more modular
                        # this is only for kubernetesengine
                        if isinstance(response, container_v1.types.ListClustersResponse):
                            targets += response.clusters
                        else:
                            targets += list(response)

                        # Remove client as it's unpickleable and adding the object to the Queue will pickle
                        # The client is later re-inserted in each Config
                        for t in targets:
                            if hasattr(t, '_client'):
                                t._client = None

                    if self.library_type == 'api_client_library':

                        # TODO need to handle long responses
                        request = method(**list_params_combination)
                        while request is not None:
                            response = request.execute()

                            if 'items' in response:
                                targets += response['items']
                            # TODO this should be more modular
                            # this is only for cloudresourcemanager
                            if 'bindings' in response:
                                targets += response['bindings']
                            # TODO this should be more modular
                            # this is only for IAM
                            if 'accounts' in response:
                                targets += response['accounts']

                            # TODO need to define the _next to handle long responses
                            # request = method_next(previous_request=request,
                            #                       previous_response=response)
                            request = None

                except HttpError as e:
                    error_json = json.loads(e.content)
                    if error_json['error']['message'] not in self.error_list:
                        self.error_list.append(error_json['error']['message'])
                        printError(error_json['error']['message'])

                except PermissionDenied as e:
                    printError("%s: %s - %s" % (e.message, self.service, self.targets))

                except Exception as e:
                    printException(e)

        except HttpError as e:
            error_json = json.loads(e.content)
            if error_json['error']['message'] not in self.error_list:
                self.error_list.append(error_json['error']['message'])
                printError(error_json['error']['message'])

        except Exception as e:
            printException(e)

        finally:
            return targets

    def _dict_product(self, d):
        keys = d.keys()
        for element in itertools.product(*d.values()):
            yield dict(zip(keys, element))



# -*- coding: utf-8 -*-

import fileinput
import os
import re

from opinel.utils.aws import get_aws_account_id
from opinel.utils.console import printDebug
from opinel.utils.credentials import read_creds

aws_dir = os.path.join(os.path.expanduser('~'), '.aws')
aws_credentials_file = os.path.join(aws_dir, 'credentials')
aws_config_file = os.path.join(aws_dir, 'config')

re_profile_name = re.compile(r'(\[(profile\s+)?(.*?)\])')

class AWSProfile(object):

    def __init__(self, filename = None, raw_profile = None, name = None, credentials = None, account_id = None):
        self.filename = filename
        self.raw_profile = raw_profile
        self.name = name
        self.account_id = account_id
        self.attributes = {}
        if self.raw_profile:
            self.parse_raw_profile()


    def get_credentials(self):
        # For now, use the existing code...
        self.credentials = read_creds(self.name)
        try:
            self.account_id = get_aws_account_id(self.credentials)
        except:
            pass
        return self.credentials


    def set_attribute(self, attribute, value):
        self.attributes[attribute] = value


    def parse_raw_profile(self):
        for line in self.raw_profile.split('\n')[1:]:
            line = line.strip()
            if line:
                values = line.split('=')
                attribute = values[0].strip()
                value = ''.join(values[1:]).strip()
                self.attributes[attribute] = value


    def write(self):
        tmp = AWSProfiles.get(self.name, quiet = True)
        if not self.raw_profile:
            self.raw_profile = tmp[0].raw_profile if len(tmp) else None
        if not self.filename:
            self.filename = tmp[0].filename if len(tmp) else self.filename
        if not self.raw_profile:
            if 'role_arn' in self.attributes and 'source_profile' in self.attributes:
                self.filename = aws_config_file
                new_raw_profile = '\n[profile %s]' % self.name
            else:
                self.filename = aws_credentials_file
                new_raw_profile = '\n[%s]' % self.name
            for attribute in self.attributes:
                new_raw_profile += '\n%s=%s' % (attribute, self.attributes[attribute])
            with open(self.filename, 'a') as f:
                f.write(new_raw_profile)
        else:
            new_raw_profile = ''
            for line in self.raw_profile.splitlines():
                line_updated = False
                for attribute in self.attributes:
                    if line.startswith(attribute):
                        new_raw_profile += '%s=%s\n' % (attribute, self.attributes[attribute])
                        line_updated = True
                        break
                if not line_updated:
                    new_raw_profile += '%s\n' % line
            with open(self.filename, 'rt') as f:
                contents = f.read()
            contents = contents.replace(self.raw_profile, new_raw_profile)
            with open(self.filename, 'wt') as f:
                f.write(contents)



class AWSProfiles(object):

    @staticmethod
    def list(names = []):
        """
        @brief

        :return:                        List of all profile names found in .aws/config and .aws/credentials
        """
        return [p.name for p in AWSProfiles.get(names)]


    @staticmethod
    def get(names = [], quiet = False):
        """
        """
        profiles = []
        profiles += AWSProfiles.find_profiles_in_file(aws_credentials_file, names, quiet)
        profiles += AWSProfiles.find_profiles_in_file(aws_config_file, names, quiet)
        return profiles


    @staticmethod
    def find_profiles_in_file(filename, names = [], quiet = True):
        profiles = []
        if type(names) != list:
            names = [ names ]
        if not quiet:
            printDebug('Searching for profiles matching %s in %s ... ' % (str(names), filename))
        name_filters = []
        for name in names:
            name_filters.append(re.compile('^%s$' % name))
        if os.path.isfile(filename):
            with open(filename, 'rt') as f:
                aws_credentials = f.read()
                existing_profiles = re_profile_name.findall(aws_credentials)
                profile_count = len(existing_profiles) - 1
                for i, profile in enumerate(existing_profiles):
                    matching_profile = False
                    raw_profile = None
                    for name_filter in name_filters:
                        if name_filter.match(profile[2]):
                            matching_profile = True
                            i1 = aws_credentials.index(profile[0])
                            if i < profile_count:
                                i2 = aws_credentials.index(existing_profiles[i+1][0])
                                raw_profile = aws_credentials[i1:i2]
                            else:
                                raw_profile = aws_credentials[i1:]
                    if len(name_filters) == 0 or matching_profile:
                        profiles.append(AWSProfile(filename = filename, raw_profile = raw_profile, name = profile[2]))
        return profiles




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os

from ScoutSuite import DEFAULT_REPORT_DIR

class ScoutSuiteArgumentParser:

    def __init__(self):
        self.parser = argparse.ArgumentParser()

        # People will still be able to use the old --provider syntax
        self.parser.add_argument("--provider", action='store_true', dest='sinkhole', help=argparse.SUPPRESS)

        self.common_providers_args_parser = argparse.ArgumentParser(add_help=False)

        self.subparsers = self.parser.add_subparsers(title="The provider you want to run scout against",
                                                     dest="provider")

        self._init_common_args_parser()

        self._init_aws_parser()
        self._init_gcp_parser()
        self._init_azure_parser()

    def _init_aws_parser(self):
        aws_parser = self.subparsers.add_parser("aws",
                                                parents=[self.common_providers_args_parser],
                                                help="Run Scout against an Amazon web Services account")

        parser = aws_parser.add_argument_group('Authentication parameters')

        default_profile = os.environ.get('AWS_PROFILE', 'default')
        default_profile_origin = " (from AWS_PROFILE)." if 'AWS_PROFILE' in os.environ else "."
        parser.add_argument('-p',
                            '--profile',
                            dest='profile',
                            default=[default_profile],
                            nargs='+',
                            help='Name of the profile. Defaults to %(default)s' + default_profile_origin)
        parser.add_argument('-c',
                            '--csv-credentials',
                            dest='csv_credentials',
                            default=None,
                            help='Path to a CSV file containing the access key ID and secret key')
        parser.add_argument('--mfa-serial',
                            dest='mfa_serial',
                            default=None,
                            help='ARN of the user\'s MFA device')
        parser.add_argument('--mfa-code',
                            dest='mfa_code',
                            default=None,
                            help='Six-digit code displayed on the MFA device.')

        parser = aws_parser.add_argument_group('Additional arguments')

        parser.add_argument('-r',
                            '--regions',
                            dest='regions',
                            default=[],
                            nargs='+',
                            help='Name of regions to run the tool in, defaults to all')
        parser.add_argument('--vpc',
                            dest='vpc',
                            default=[],
                            nargs='+',
                            help='Name of VPC to run the tool in, defaults to all')
        parser.add_argument('--ip-ranges',
                            dest='ip_ranges',
                            default=[],
                            nargs='+',
                            help='Config file(s) that contain your known IP ranges')
        parser.add_argument('--ip-ranges-name-key',
                            dest='ip_ranges_name_key',
                            default='name',
                            help='Name of the key containing the display name of a known CIDR')

    def _init_gcp_parser(self):
        gcp_parser = self.subparsers.add_parser("gcp",
                                                parents=[self.common_providers_args_parser],
                                                help="Run Scout against a Google Cloud Platform account")

        parser = gcp_parser.add_argument_group('Authentication modes')
        gcp_auth_modes = parser.add_mutually_exclusive_group(required=True)

        gcp_auth_modes.add_argument('-u',
                                    '--user-account',
                                    action='store_true',
                                    help='Run Scout with a Google Account')

        gcp_auth_modes.add_argument('-s',
                                    '--service-account',
                                    action='store',
                                    metavar="KEY_FILE",
                                    help='Run Scout with a Google Service Account with the specified '
                                         'Google Service Account Application Credentials file')

        gcp_scope = gcp_parser.add_argument_group('Additional arguments')

        gcp_scope.add_argument('--project-id',
                               action='store',
                               help='ID of the GCP Project to scan')

        gcp_scope.add_argument('--folder-id',
                               action='store',
                               help='ID of the GCP Folder to scan')

        gcp_scope.add_argument('--organization-id',
                               action='store',
                               help='ID of the GCP Organization to scan')

        gcp_scope.add_argument('--all-projects',
                               action='store_true',
                               help='Scan all of the accessible projects')

    def _init_azure_parser(self):
        parser = self.subparsers.add_parser("azure",
                                            parents=[self.common_providers_args_parser],
                                            help="Run Scout against a Microsoft Azure account")

        azure_parser = parser.add_argument_group('Authentication modes')
        azure_auth_params = parser.add_argument_group('Authentication parameters')

        azure_auth_modes = azure_parser.add_mutually_exclusive_group(required=True)

        azure_auth_modes.add_argument('-c',
                                      '--cli',
                                      action='store_true',
                                      help='Run Scout using configured azure-cli credentials')

        azure_auth_modes.add_argument('-m',
                                      '--msi',
                                      action='store_true',
                                      help='Run Scout with Managed Service Identity')

        azure_auth_modes.add_argument('-s',
                                      '--service-principal',
                                      action='store_true',
                                      help='Run Scout with an Azure Service Principal')
        azure_auth_params.add_argument('--tenant',
                                       action='store',
                                       dest='tenant_id',
                                       help='Tenant ID of the service principal')
        azure_auth_params.add_argument('--subscription',
                                       action='store',
                                       dest='subscription_id',
                                       help='Subscription ID of the service principal')
        azure_auth_params.add_argument('--client-id',
                                       action='store',
                                       dest='client_id',
                                       help='Client ID of the service principal')
        azure_auth_params.add_argument('--client-secret',
                                       action='store',
                                       dest='client_secret',
                                       help='Client of the service principal')

        azure_auth_modes.add_argument('--file-auth',
                                      action='store',
                                      type=argparse.FileType('r'),
                                      dest='file_auth',
                                      metavar="FILE",
                                      help='Run Scout with the specified credential file')

        azure_auth_modes.add_argument('--user-account',
                                      action='store_true',
                                      help='Run Scout with user credentials')
        azure_auth_params.add_argument('-u',
                                       '--username',
                                       action='store',
                                       default=None,
                                       dest='username',
                                       help='Username of the Azure account')
        azure_auth_params.add_argument('-p',
                                       '--password',
                                       action='store',
                                       default=None,
                                       dest='password',
                                       help='Password of the Azure account')

    def _init_common_args_parser(self):
        parser = self.common_providers_args_parser.add_argument_group('Scout Arguments')

        parser.add_argument('-f',
                            '--force',
                            dest='force_write',
                            default=False,
                            action='store_true',
                            help='Overwrite existing files')
        parser.add_argument('-l', '--local',
                            dest='fetch_local',
                            default=False,
                            action='store_true',
                            help='Use local data previously fetched and re-run the analysis.')
        parser.add_argument('--debug',
                            dest='debug',
                            default=False,
                            action='store_true',
                            help='Print the stack trace when exception occurs')
        parser.add_argument('--resume',
                            dest='resume',
                            default=False,
                            action='store_true',
                            help='Complete a partial (throttled) run')
        parser.add_argument('--update',
                            dest='update',
                            default=False,
                            action='store_true',
                            help='Reload all the existing data and only overwrite data in scope for this run')
        parser.add_argument('--ruleset',
                            dest='ruleset',
                            default='default.json',
                            nargs='?',
                            help='Set of rules to be used during the analysis.')
        parser.add_argument('--no-browser',
                            dest='no_browser',
                            default=False,
                            action='store_true',
                            help='Do not automatically open the report in the browser.')
        parser.add_argument('--thread-config',
                            dest='thread_config',
                            type=int,
                            default=4,
                            help='Level of multi-threading wanted [1-5]; defaults to 4.')
        parser.add_argument('--report-dir',
                            dest='report_dir',
                            default=DEFAULT_REPORT_DIR,
                            help='Path of the Scout report.')
        parser.add_argument('--timestamp',
                            dest='timestamp',
                            default=False,
                            nargs='?',
                            help='Timestamp added to the name of the report (default is current time in UTC).')
        parser.add_argument('--services',
                            dest='services',
                            default=[],
                            nargs='+',
                            help='Name of in-scope services.')
        parser.add_argument('--skip',
                            dest='skipped_services',
                            default=[],
                            nargs='+',
                            help='Name of out-of-scope services.')
        parser.add_argument('--exceptions',
                            dest='exceptions',
                            default=[None],
                            nargs='+',
                            help='Exception file to use during analysis.')

    def parse_args(self, args=None):
        args = self.parser.parse_args(args)

        # Cannot simply use required for backward compatibility
        if not args.provider:
            self.parser.error('You need to input a provider')
        # If local analysis, overwrite results
        if args.__dict__.get('fetch_local'):
            args.force_write = True
        return args



# Import future print
from __future__ import print_function

import boto3
import datetime
import dateutil.parser
import json
import fileinput
import os
import re
import requests # TODO: get rid of that and make sure urllib2 validates certs ?
import string

from opinel.utils.console import printException, printError, printInfo
from opinel.utils.console import prompt_4_mfa_code
from opinel.utils.fs import save_blob_as_json
from opinel.utils.aws import connect_service


########################################
# Globals
########################################

re_profile_name = re.compile(r'\[(.*)\]')
re_access_key = re.compile(r'aws_access_key_id')
re_secret_key = re.compile(r'aws_secret_access_key')
re_role_arn = re.compile(r'role_arn')
re_session_token = re.compile(r'aws_session_token')
re_security_token = re.compile(r'aws_security_token')
re_expiration = re.compile(r'expiration')
re_source_profile = re.compile(r'source_profile')
re_external_id = re.compile(r'aws_external_id')

re_gov_region = re.compile(r'(.*?)-gov-(.*?)')
re_cn_region = re.compile(r'^cn-(.*?)')

re_port_range = re.compile(r'(\d+)\-(\d+)')
re_single_port = re.compile(r'(\d+)')

mfa_serial = r'(aws_mfa_serial|mfa_serial)'
mfa_serial_format = r'arn:aws:iam::\d+:mfa/[a-zA-Z0-9\+=,.@_-]+'
re_mfa_serial = re.compile(mfa_serial)
re_mfa_serial_format = re.compile(mfa_serial_format)


aws_config_dir = os.path.join(os.path.expanduser('~'), '.aws')
aws_credentials_file = os.path.join(aws_config_dir, 'credentials')
aws_credentials_file_tmp = os.path.join(aws_config_dir, 'credentials.tmp')
aws_config_file = os.path.join(aws_config_dir, 'config')


########################################
# Credentials read/write functions
########################################


def assume_role(role_name, credentials, role_arn, role_session_name, silent = False):
    """
    Assume role and save credentials

    :param role_name:
    :param credentials:
    :param role_arn:
    :param role_session_name:
    :param silent:
    :return:
    """
    external_id = credentials.pop('ExternalId') if 'ExternalId' in credentials else None
    # Connect to STS
    sts_client = connect_service('sts', credentials, silent = silent)
    # Set required arguments for assume role call
    sts_args = {
      'RoleArn': role_arn,
      'RoleSessionName': role_session_name
    }
    # MFA used ?
    if 'mfa_serial' in credentials and 'mfa_code' in credentials:
      sts_args['TokenCode'] = credentials['mfa_code']
      sts_args['SerialNumber'] = credentials['mfa_serial']
    # External ID used ?
    if external_id:
      sts_args['ExternalId'] = external_id
    # Assume the role
    sts_response = sts_client.assume_role(**sts_args)
    credentials = sts_response['Credentials']
    cached_credentials_filename = get_cached_credentials_filename(role_name, role_arn)
    #with open(cached_credentials_filename, 'wt+') as f:
    #   write_data_to_file(f, sts_response, True, False)
    cached_credentials_path = os.path.dirname(cached_credentials_filename)
    if not os.path.isdir(cached_credentials_path):
        os.makedirs(cached_credentials_path)
    save_blob_as_json(cached_credentials_filename, sts_response, True, False) # blob, force_write, debug):
    return credentials


def get_cached_credentials_filename(role_name, role_arn):
    """
    Construct filepath for cached credentials (AWS CLI scheme)

    :param role_name:
    :param role_arn:
    :return:
    """
    filename_p1 = role_name.replace('/','-')
    filename_p2 = role_arn.replace('/', '-').replace(':', '_')
    return os.path.join(os.path.join(os.path.expanduser('~'), '.aws'), 'cli/cache/%s--%s.json' %
                        (filename_p1, filename_p2))


def get_profiles_from_aws_credentials_file(credentials_files = [aws_credentials_file, aws_config_file]):
    """

    :param credentials_files:

    :return:
    """
    profiles = []
    for filename in credentials_files:
        if os.path.isfile(filename):
            with open(filename) as f:
                lines = f.readlines()
                for line in lines:
                    groups = re_profile_name.match(line)
                    if groups:
                        profiles.append(groups.groups()[0])
    return sorted(profiles)


def generate_password(length=16):
    """
    Generate a password using random characters from uppercase, lowercase, digits, and symbols

    :param length:                      Length of the password to be generated
    :return:                            The random password
    """
    chars = string.ascii_letters + string.digits + '!@#$%^&*()_+-=[]{};:,<.>?|'
    modulus = len(chars)
    pchars = os.urandom(16)
    if type(pchars) == str:
        return ''.join(chars[i % modulus] for i in map(ord, pchars))
    else:
        return ''.join(chars[i % modulus] for i in pchars)


def init_creds():
    """
    Create a dictionary with all the necessary keys set to "None"

    :return:
    """
    return { 'AccessKeyId': None, 'SecretAccessKey': None, 'SessionToken': None,
             'Expiration': None, 'SerialNumber': None, 'TokenCode': None }


def init_sts_session(profile_name, credentials, duration = 28800, session_name = None, save_creds = True):
    """
    Fetch STS credentials

    :param profile_name:
    :param credentials:
    :param duration:
    :param session_name:
    :param save_creds:
    :return:
    """
    # Set STS arguments
    sts_args = {
        'DurationSeconds': duration
    }
    # Prompt for MFA code if MFA serial present
    if 'SerialNumber' in credentials and credentials['SerialNumber']:
        if not credentials['TokenCode']:
            credentials['TokenCode'] = prompt_4_mfa_code()
            if credentials['TokenCode'] == 'q':
                credentials['SerialNumber'] = None
        sts_args['TokenCode'] = credentials['TokenCode']
        sts_args['SerialNumber'] = credentials['SerialNumber']
    # Init session
    sts_client = boto3.session.Session(credentials['AccessKeyId'], credentials['SecretAccessKey']).client('sts')
    sts_response = sts_client.get_session_token(**sts_args)
    if save_creds:
        # Move long-lived credentials if needed
        if not profile_name.endswith('-nomfa') and credentials['AccessKeyId'].startswith('AKIA'):
            write_creds_to_aws_credentials_file(profile_name + '-nomfa', credentials)
        # Save STS values in the .aws/credentials file
        write_creds_to_aws_credentials_file(profile_name, sts_response['Credentials'])
    return sts_response['Credentials']


def read_creds_from_aws_credentials_file(profile_name, credentials_file = aws_credentials_file):
    """
    Read credentials from AWS config file

    :param profile_name:
    :param credentials_file:
    :return:
    """
    credentials = init_creds()
    profile_found = False
    try:
        # Make sure the ~.aws folder exists
        if not os.path.exists(aws_config_dir):
            os.makedirs(aws_config_dir)
        with open(credentials_file, 'rt') as cf:
            for line in cf:
                profile_line = re_profile_name.match(line)
                if profile_line:
                    if profile_line.groups()[0] == profile_name:
                        profile_found = True
                    else:
                        profile_found = False
                if profile_found:
                    if re_access_key.match(line):
                        credentials['AccessKeyId'] = line.split("=")[1].strip()
                    elif re_secret_key.match(line):
                        credentials['SecretAccessKey'] = line.split("=")[1].strip()
                    elif re_mfa_serial.match(line):
                        credentials['SerialNumber'] = (line.split('=')[1]).strip()
                    elif re_session_token.match(line) or re_security_token.match(line):
                        credentials['SessionToken'] = ('='.join(x for x in line.split('=')[1:])).strip()
                    elif re_expiration.match(line):
                        credentials['Expiration'] = ('='.join(x for x in line.split('=')[1:])).strip()
    except Exception as e:
        # Silent if error is due to no ~/.aws/credentials file
        if not hasattr(e, 'errno') or e.errno != 2:
            printException(e)
    return credentials


def read_creds_from_csv(filename):
    """
    Read credentials from a CSV file

    :param filename:
    :return:
    """
    key_id = None
    secret = None
    mfa_serial = None
    secret_next = False
    with open(filename, 'rt') as csvfile:
        for i, line in enumerate(csvfile):
            values = line.split(',')
            for v in values:
                if v.startswith('AKIA'):
                    key_id = v.strip()
                    secret_next = True
                elif secret_next:
                    secret = v.strip()
                    secret_next = False
                elif re_mfa_serial_format.match(v):
                    mfa_serial = v.strip()
    return key_id, secret, mfa_serial


def read_creds_from_ec2_instance_metadata():
    """
    Read credentials from EC2 instance metadata (IAM role)

    :return:
    """
    creds = init_creds()
    try:
        has_role = requests.get('http://169.254.169.254/latest/meta-data/iam/security-credentials', timeout = 1)
        if has_role.status_code == 200:
            iam_role = has_role.text
            credentials = requests.get('http://169.254.169.254/latest/meta-data/iam/security-credentials/%s/' %
                                       iam_role.strip()).json()
            for c in ['AccessKeyId', 'SecretAccessKey']:
                creds[c] = credentials[c]
            creds['SessionToken'] = credentials['Token']
        return creds
    except Exception as e:
        return False


def read_creds_from_ecs_container_metadata():
    """
    Read credentials from ECS instance metadata (IAM role)

    :return:
    """
    creds = init_creds()
    try:
        ecs_metadata_relative_uri = os.environ['AWS_CONTAINER_CREDENTIALS_RELATIVE_URI']
        credentials = requests.get('http://169.254.170.2' + ecs_metadata_relative_uri, timeout = 1).json()
        for c in ['AccessKeyId', 'SecretAccessKey']:
            creds[c] = credentials[c]
            creds['SessionToken'] = credentials['Token']
        return creds
    except Exception as e:
        return False


def read_creds_from_environment_variables():
    """
    Read credentials from environment variables

    :return:
    """
    creds = init_creds()
    # Check environment variables
    if 'AWS_ACCESS_KEY_ID' in os.environ and 'AWS_SECRET_ACCESS_KEY' in os.environ:
        creds['AccessKeyId'] = os.environ['AWS_ACCESS_KEY_ID']
        creds['SecretAccessKey'] = os.environ['AWS_SECRET_ACCESS_KEY']
        if 'AWS_SESSION_TOKEN' in os.environ:
            creds['SessionToken'] = os.environ['AWS_SESSION_TOKEN']
    return creds


def read_profile_from_environment_variables():
    """
    Read profiles from env

    :return:
    """
    role_arn = os.environ.get('AWS_ROLE_ARN', None)
    external_id = os.environ.get('AWS_EXTERNAL_ID', None)
    return role_arn, external_id


def read_profile_from_aws_config_file(profile_name, config_file = aws_config_file):
    """
    Read profiles from AWS config file

    :param profile_name:
    :param config_file:
    :return:
    """
    role_arn = None
    source_profile = 'default'
    mfa_serial = None
    profile_found = False
    external_id = None
    try:
        with open(config_file, 'rt') as config:
            for line in config:
                profile_line = re_profile_name.match(line)
                if profile_line:
                    role_profile_name = profile_line.groups()[0].split()[-1]
                    if role_profile_name == profile_name:
                        profile_found = True
                    else:
                        profile_found = False
                if profile_found:
                    if re_role_arn.match(line):
                        role_arn = line.split('=')[1].strip()
                    elif re_source_profile.match(line):
                        source_profile = line.split('=')[1].strip()
                    elif re_mfa_serial.match(line):
                        mfa_serial = line.split('=')[1].strip()
                    elif re_external_id.match(line):
                        external_id = line.split('=')[1].strip()
    except Exception as e:
        # Silent if error is due to no .aws/config file
        if not hasattr(e, 'errno') or e.errno != 2:
            printException(e)
    return role_arn, source_profile, mfa_serial, external_id


def show_profiles_from_aws_credentials_file(credentials_files = [aws_credentials_file, aws_config_file]):
    """
    Show profile names from ~/.aws/credentials

    :param credentials_files:
    :return:
    """
    profiles = get_profiles_from_aws_credentials_file(credentials_files)
    for profile in set(profiles):
        printInfo(' * %s' % profile)


def write_creds_to_aws_credentials_file(profile_name, credentials, credentials_file = aws_credentials_file):
    """
    Write credentials to AWS config file

    :param profile_name:
    :param credentials:
    :param credentials_file:
    :return:
    """
    profile_found = False
    profile_ever_found = False
    session_token_written = False
    security_token_written = False
    mfa_serial_written = False
    expiration_written = False
    # Create the .aws folder if needed
    if not os.path.isdir(aws_config_dir):
        os.mkdir(aws_config_dir)
    # Create an empty file if target does not exist
    if not os.path.isfile(credentials_file):
        open(credentials_file, 'a').close()
    # Open and parse/edit file
    for line in fileinput.input(credentials_file, inplace=True):
        profile_line = re_profile_name.match(line)
        if profile_line:
            if profile_line.groups()[0] == profile_name:
                profile_found = True
                profile_ever_found = True
            else:
                profile_found = False
            print(line.rstrip())
        elif profile_found:
            if re_access_key.match(line) and 'AccessKeyId' in credentials and credentials['AccessKeyId']:
                print('aws_access_key_id = %s' % credentials['AccessKeyId'])
            elif re_secret_key.match(line) and 'SecretAccessKey' in credentials and credentials['SecretAccessKey']:
                print('aws_secret_access_key = %s' % credentials['SecretAccessKey'])
            elif re_mfa_serial.match(line) and 'SerialNumber' in credentials and credentials['SerialNumber']:
                print('aws_mfa_serial = %s' % credentials['SerialNumber'])
                mfa_serial_written = True
            elif re_session_token.match(line) and 'SessionToken' in credentials and credentials['SessionToken']:
                print('aws_session_token = %s' % credentials['SessionToken'])
                session_token_written = True
            elif re_security_token.match(line) and 'SessionToken' in credentials and credentials['SessionToken']:
                print('aws_security_token = %s' % credentials['SessionToken'])
                security_token_written = True
            elif re_expiration.match(line) and 'Expiration' in credentials and credentials['Expiration']:
                print('expiration = %s' % credentials['Expiration'])
                expiration_written = True
            else:
                print(line.rstrip())
        else:
            print(line.rstrip())

    # Complete the profile if needed
    if profile_found:
        with open(credentials_file, 'a') as f:
            complete_profile(f, credentials, session_token_written, mfa_serial_written)

    # Add new profile if not found
    if not profile_ever_found:
        with open(credentials_file, 'a') as f:
            f.write('[%s]\n' % profile_name)
            f.write('aws_access_key_id = %s\n' % credentials['AccessKeyId'])
            f.write('aws_secret_access_key = %s\n' % credentials['SecretAccessKey'])
            complete_profile(f, credentials, session_token_written, mfa_serial_written)


def complete_profile(f, credentials, session_token_written, mfa_serial_written):
    """
    Append session token and mfa serial if needed

    :param f:
    :param credentials:
    :param session_token_written:
    :param mfa_serial_written:
    :return:
    """
    session_token = credentials['SessionToken'] if 'SessionToken' in credentials else None
    mfa_serial = credentials['SerialNumber'] if 'SerialNumber' in credentials else None
    if session_token and not session_token_written:
        f.write('aws_session_token = %s\n' % session_token)
    if mfa_serial and not mfa_serial_written:
        f.write('aws_mfa_serial = %s\n' % mfa_serial)

########################################
# Main function
########################################


def read_creds(profile_name, csv_file = None, mfa_serial_arg = None, mfa_code = None, force_init = False,
               role_session_name = 'opinel'):
    """
    Read credentials from anywhere (CSV, Environment, Instance metadata, config/credentials)

    :param profile_name:
    :param csv_file:
    :param mfa_serial_arg:
    :param mfa_code:
    :param force_init:
    :param role_session_name:

    :return:
    """
    first_sts_session = False
    source_profile = None
    role_mfa_serial = None
    expiration = None
    credentials = init_creds()
    role_arn, external_id = read_profile_from_environment_variables()
    if csv_file:
        # Read credentials from a CSV file that was provided
        credentials['AccessKeyId'], credentials['SecretAccessKey'], credentials['SerialNumber'] = \
            read_creds_from_csv(csv_file)
    elif profile_name == 'default':
        # Try reading credentials from environment variables (Issue #11) if the profile name is 'default'
        credentials = read_creds_from_environment_variables()
    if ('AccessKeyId' not in credentials or not credentials['AccessKeyId']) \
            and not csv_file and profile_name == 'default':
        ec2_credentials = read_creds_from_ec2_instance_metadata()
        if ec2_credentials:
            credentials = ec2_credentials
        else:
            ecs_credentials = read_creds_from_ecs_container_metadata()
            if ecs_credentials:
                credentials = ecs_credentials
        # TODO support lambda
    if role_arn or (not credentials['AccessKeyId'] and not csv_file):
        # Lookup if a role is defined in ~/.aws/config
        if not role_arn:
            role_arn, source_profile, role_mfa_serial, external_id = read_profile_from_aws_config_file(profile_name)
        # Scout2 issue 237 - credentials file may be used to configure role-based profiles...
        if not role_arn:
            role_arn, source_profile, role_mfa_serial, external_id = \
                read_profile_from_aws_config_file(profile_name, config_file = aws_credentials_file)
        if role_arn:
            # Lookup cached credentials
            try:
                cached_credentials_filename = get_cached_credentials_filename(profile_name, role_arn)
                with open(cached_credentials_filename, 'rt') as f:
                    assume_role_data = json.load(f)
                    oldcred = credentials
                    credentials = assume_role_data['Credentials']
                    expiration = dateutil.parser.parse(credentials['Expiration'])
                    expiration = expiration.replace(tzinfo=None)
                    current = datetime.datetime.utcnow()
                    if expiration < current:
                        print('Role\'s credentials have expired on %s' % credentials['Expiration'])
                        credentials = oldcred
            except Exception as e:
                pass
            if not expiration or expiration < current or credentials['AccessKeyId'] == None:
                if source_profile:
                    credentials = read_creds(source_profile)
                if role_mfa_serial:
                    credentials['SerialNumber'] = role_mfa_serial
                    # Auto prompt for a code...
                    if not mfa_code:
                        credentials['TokenCode'] = prompt_4_mfa_code()
                if external_id:
                    credentials['ExternalId'] = external_id
                credentials = assume_role(profile_name, credentials, role_arn, role_session_name)
        # Read from ~/.aws/credentials
        else:
            credentials = read_creds_from_aws_credentials_file(profile_name)
            if credentials['SessionToken']:
                if 'Expiration' in credentials and credentials['Expiration']:
                    expiration = dateutil.parser.parse(credentials['Expiration'])
                    expiration = expiration.replace(tzinfo=None)
                    current = datetime.datetime.utcnow()
                    if expiration < current:
                        printInfo('Saved STS credentials expired on %s' % credentials['Expiration'])
                        force_init = True
                else:
                    force_init = True
                    sts_credentials = credentials
            else:
                first_sts_session = True
            if force_init or (mfa_serial_arg and mfa_code):
                credentials = read_creds_from_aws_credentials_file(profile_name if first_sts_session
                                                                   else '%s-nomfa' % profile_name)
                if not credentials['AccessKeyId']:
                    printInfo('Warning: Unable to determine STS token expiration; later API calls may fail.')
                    credentials = sts_credentials
                else:
                    if mfa_serial_arg:
                        credentials['SerialNumber'] = mfa_serial_arg
                    if mfa_code:
                        credentials['TokenCode'] = mfa_code
                    if 'AccessKeyId' in credentials and credentials['AccessKeyId']:
                        credentials = init_sts_session(profile_name, credentials)
    # If we don't have valid creds by now, print an error message
    if 'AccessKeyId' not in credentials or credentials['AccessKeyId'] == None or \
            'SecretAccessKey' not in credentials or credentials['SecretAccessKey'] == None:
        printError('Error: could not find AWS credentials. Use the --help option for more information.')
    if not 'AccessKeyId' in credentials:
        credentials = { 'AccessKeyId': None }
    return credentials



# -*- coding: utf-8 -*-

import os
import re
import sys

from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.base.configs.browser import get_value_at


########################################
# Functions
########################################

def format_listall_output(format_file, format_item_dir, format, rule, option_prefix=None, template=None,
                          skip_options=False):
    """
    Prepare listall output template

    :param format_file:
    :param format_item_dir:
    :param format:
    :param config:
    :param option_prefix:
    :param template:
    :param skip_options:
    :return:
    """
    # Set the list of keys if printing from a file spec
    # _LINE_(whatever)_EOL_
    # _ITEM_(resource)_METI_
    # _KEY_(path_to_value)
    if format_file and os.path.isfile(format_file):
        if not template:
            with open(format_file, 'rt') as f:
                template = f.read()
        # Optional files
        if not skip_options:
            re_option = re.compile(r'(%_OPTION_\((.*?)\)_NOITPO_)')
            optional_files = re_option.findall(template)
            for optional_file in optional_files:
                if optional_file[1].startswith(option_prefix + '-'):
                    with open(os.path.join(format_item_dir, optional_file[1].strip()), 'rt') as f:
                        template = template.replace(optional_file[0].strip(), f.read())
        # Include files if needed
        re_file = re.compile(r'(_FILE_\((.*?)\)_ELIF_)')
        while True:
            requested_files = re_file.findall(template)
            available_files = os.listdir(format_item_dir) if format_item_dir else []
            for requested_file in requested_files:
                if requested_file[1].strip() in available_files:
                    with open(os.path.join(format_item_dir, requested_file[1].strip()), 'rt') as f:
                        template = template.replace(requested_file[0].strip(), f.read())
            # Find items and keys to be printed
            re_line = re.compile(r'(_ITEM_\((.*?)\)_METI_)')
            re_key = re.compile(r'_KEY_\(*(.*?)\)', re.DOTALL | re.MULTILINE)  # Remove the multiline ?
            lines = re_line.findall(template)
            for (i, line) in enumerate(lines):
                lines[i] = line + (re_key.findall(line[1]),)
            requested_files = re_file.findall(template)
            if len(requested_files) == 0:
                break
    elif format and format[0] == 'csv':
        keys = rule.keys
        line = ', '.join('_KEY_(%s)' % k for k in keys)
        lines = [(line, line, keys)]
        template = line
    return (lines, template)


def generate_listall_output(lines, resources, aws_config, template, arguments, nodup=False):
    """
    Format and print the output of ListAll

    :param lines:
    :param resources:
    :param aws_config:
    :param template:
    :param arguments:
    :param nodup:
    :return:
    """
    for line in lines:
        output = []
        for resource in resources:
            current_path = resource.split('.')
            outline = line[1]
            for key in line[2]:
                outline = outline.replace('_KEY_(' + key + ')',
                                          get_value_at(aws_config['services'], current_path, key, True))
            output.append(outline)
        output = '\n'.join(line for line in sorted(set(output)))
        template = template.replace(line[0], output)
    for (i, argument) in enumerate(arguments):
        template = template.replace('_ARG_%d_' % i, argument)
    return template


########################################
# Status updates
########################################

class FetchStatusLogger:

    def __init__(self, targets, add_regions=False):
        self.targets = []
        self.formatted_string = '\r '
        self.counts = {}
        target_names = ()
        if add_regions:
            targets = ('regions',) + targets
        for target in targets:
            target_type = target[0] if type(target) == tuple else target
            self.targets.append(target_type)
            manage_dictionary(self.counts, target_type, {'discovered': 0, 'fetched': 0})
            target_names += (target_type,)
            self.formatted_string += ' %18s'
        self.__out(target_names, True)

    def show(self, new_line=False):
        values = ()
        for target in self.targets:
            v = '%s/%s' % (self.counts[target]['fetched'], self.counts[target]['discovered'])
            values += (v,)
        self.__out(values, new_line)

    def __out(self, values, new_line):
        try:
            sys.stdout.write(self.formatted_string % values)
            sys.stdout.flush()
            if new_line:
                sys.stdout.write('\n')
        except:
            pass



# -*- coding: utf-8 -*-
from __future__ import print_function

import datetime
import json
import os
import yaml

from opinel.utils.console import printError, printException, prompt_4_overwrite
from opinel.utils.conditions import pass_condition



class CustomJSONEncoder(json.JSONEncoder):
    """
    JSON encoder class
    """
    def default(self, o):
        if type(o) == datetime.datetime:
            return str(o)
        else:
            return o.__dict__


def load_data(data_file, key_name = None, local_file = False, format = 'json'):
    """
    Load a JSON data file

    :param data_file:
    :param key_name:
    :param local_file:
    :return:
    """
    if local_file:
        if data_file.startswith('/'):
            src_file = data_file
        else:
            src_dir = os.getcwd()
            src_file = os.path.join(src_dir, data_file)
    else:
        src_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data')
        if not os.path.isdir(src_dir):
            src_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../data')
        src_file = os.path.join(src_dir, data_file)
    with open(src_file) as f:
        if format == 'json':
            data = json.load(f)
        elif format == 'yaml':
            data = yaml.load(f)
        elif format not in ['json', 'yaml'] and not key_name:
            data = f.read()
        else:
            printError('Error, argument \'key_name\' may not be used with data in %s format.' % format)
            return None
    if key_name:
        data = data[key_name]
    return data


def read_ip_ranges(filename, local_file = True, ip_only = False, conditions = []):
    """
    Returns the list of IP prefixes from an ip-ranges file

    :param filename:
    :param local_file:
    :param conditions:
    :param ip_only:
    :return:
    """
    targets = []
    data = load_data(filename, local_file = local_file)
    if 'source' in data:
        # Filtered IP ranges
        conditions = data['conditions']
        local_file = data['local_file'] if 'local_file' in data else False
        data = load_data(data['source'], local_file = local_file, key_name = 'prefixes')
    else:
        # Plain IP ranges
        data = data['prefixes']
    for d in data:
        condition_passed = True
        for condition in conditions:
            if type(condition) != list or len(condition) < 3:
                continue
            condition_passed = pass_condition(d[condition[0]], condition[1], condition[2])
            if not condition_passed:
                break
        if condition_passed:
            targets.append(d)
    if ip_only:
        ips = []
        for t in targets:
            ips.append(t['ip_prefix'])
        return ips
    else:
        return targets


def read_file(file_path, mode = 'rt'):
    """
    Read the contents of a file

    :param file_path:                   Path of the file to be read

    :return:                            Contents of the file
    """
    contents = ''
    with open(file_path, mode) as f:
        contents = f.read()
    return contents


def save_blob_as_json(filename, blob, force_write, debug):
    """
    Creates/Modifies file and saves python object as JSON

    :param filename:
    :param blob:
    :param force_write:
    :param debug:

    :return:
    """
    try:
        if prompt_4_overwrite(filename, force_write):
            with open(filename, 'wt') as f:
                print('%s' % json.dumps(blob, indent=4 if debug else None, separators=(',', ': '), sort_keys=True, cls=CustomJSONEncoder), file=f)
    except Exception as e:
        printException(e)
        pass


def save_ip_ranges(profile_name, prefixes, force_write, debug, output_format = 'json'):
    """
    Creates/Modifies an ip-range-XXX.json file

    :param profile_name:
    :param prefixes:
    :param force_write:
    :param debug:

    :return:
    """
    filename = 'ip-ranges-%s.json' % profile_name
    ip_ranges = {}
    ip_ranges['createDate'] = datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
    # Unique prefixes
    unique_prefixes = {}
    for prefix in prefixes:
        if type(prefix) == dict:
            unique_prefixes[prefix['ip_prefix']] = prefix
        else:
            unique_prefixes[prefix] = {'ip_prefix': prefix}
    unique_prefixes = list(unique_prefixes.values())
    ip_ranges['prefixes'] = unique_prefixes
    if output_format == 'json':
        save_blob_as_json(filename, ip_ranges, force_write, debug)
    else:
        # Write as CSV
        output = 'account_id, region, ip, instance_id, instance_name\n'
        for prefix in unique_prefixes:
            output += '%s, %s, %s, %s, %s\n' % (prefix['account_id'], prefix['region'], prefix['ip_prefix'], prefix['instance_id'], prefix['name'])
        with open('ip-ranges-%s.csv' % profile_name, 'wt') as f:
            f.write(output)



# -*- coding: utf-8 -*-

import json

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients



########################################
# SQSRegionConfig
########################################

class SQSRegionConfig(RegionConfig):
    """
    SQS configuration for a single AWS region
    """

    def parse_queue(self, global_params, region, queue_url):
        """
        Parse a single queue and fetch additional attributes

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param queue_url:               URL of the AWS queue
        """
        queue = {'QueueUrl': queue_url}
        attributes = api_clients[region].get_queue_attributes(QueueUrl = queue_url, AttributeNames = ['CreatedTimestamp', 'Policy', 'QueueArn', 'KmsMasterKeyId'])['Attributes']
        queue['arn'] = attributes.pop('QueueArn')
        queue['kms_master_key_id'] = attributes.pop('KmsMasterKeyId', None)
        queue['CreatedTimestamp'] = attributes.pop('CreatedTimestamp', None)

        if 'Policy' in attributes:
            queue['Policy'] = json.loads(attributes['Policy'])
        else:
            queue['Policy'] = {'Statement': []}

        queue['name'] = queue['arn'].split(':')[-1]
        self.queues[queue['name']] = queue



########################################
# SQSConfig
########################################

class SQSConfig(RegionalServiceConfig):
    """
    SQS configuration for all AWS regions
    """

    region_config_class = SQSRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(SQSConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

from opinel.utils.console import printError

import operator
from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig


class CloudSQLConfig(GCPBaseConfig):
    targets = (
        ('instances', 'Instances', 'list', {'project': '{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):

        self.library_type = 'api_client_library'

        self.instances = {}
        self.instances_count = 0

        # TODO figure out why GCP returns errors when running with more then 1 thread (multithreading)
        super(CloudSQLConfig, self).__init__(thread_config=1)

    def parse_instances(self, instance, params):
        """
        Parse a single Cloud Storage bucket

        :param bucket: Bucket object  representing a single bucket
        :param params: Dictionary of parameters, including API client
        :return: None
        """
        instance_dict = {}
        instance_dict['id'] = self.get_non_provider_id(instance['name'])
        instance_dict['name'] = instance['name']
        instance_dict['project_id'] = instance['project']
        instance_dict['automatic_backup_enabled'] = instance['settings']['backupConfiguration']['enabled']
        instance_dict['database_version'] = instance['databaseVersion']
        instance_dict['log_enabled'] = self._is_log_enabled(instance)
        instance_dict['ssl_required'] = self._is_ssl_required(instance)
        instance_dict['backups'] = self._get_instance_backups(instance, params)
        instance_dict['users'] = self._get_users(instance, params)
        instance_dict['authorized_networks'] = instance['settings']['ipConfiguration']['authorizedNetworks']

        instance_dict['last_backup_timestamp'] = \
            instance_dict['backups'][max(instance_dict['backups'].keys(),
                                         key=(lambda k: instance_dict['backups'][k]['creation_timestamp']))]['creation_timestamp'] if \
                instance_dict['backups'].keys() else 'N/A'

        self.instances[instance_dict['id']] = instance_dict

    def _get_users(self, instance, params):
        users_dict = {}

        try:
            users = params['api_client'].users().list(project=instance['project'], instance=instance['name']).execute()
            for user in users['items']:
                users_dict[user['name']] = self._parse_user(user)

        except Exception as e:
            printError('Failed to fetch users for SQL instance %s: %s' % (instance['name'], e))
            
        return users_dict

    def _parse_user(self, user):
        user_dict = {}
        user_dict['name'] = user['name']
        user_dict['host'] = user.get('host')
        return user_dict

    def _get_instance_backups(self, instance, params):

        backups_dict = {}

        try:
            backups = params['api_client'].backupRuns().list(project=instance['project'],
                                                             instance=instance['name']).execute()
        # this is triggered when there are no backups
        except AttributeError as e:
            return backups_dict
        except Exception as e:
            printError('Failed to fetch backups for SQL instance %s: %s' % (instance['name'], e))
            return backups_dict

        try:
            if 'items' in backups:
                for backup in backups['items']:
                    if backup['status'] == 'SUCCESSFUL':
                        backups_dict[backup['id']] = {
                            'backup_url': backup['selfLink'],
                            'creation_timestamp': backup['endTime'],
                            'status': backup['status'],
                            'type': backup['type']
                        }

            return backups_dict

        except Exception as e:
            printError('Failed to parse backups for SQL instance %s: %s' % (instance['name'], e))
            return None

    def _is_log_enabled(self, instance) :
        return instance['settings']['backupConfiguration'].get('binaryLogEnabled')

    def _is_ssl_required(self, instance):
        return instance['settings']['ipConfiguration'].get('requireSsl')


# -*- coding: utf-8 -*-
from __future__ import print_function

import datetime
import os
import shutil
import zipfile

import dateutil.tz
from opinel.utils.console import printInfo, printException

from ScoutSuite import AWSCONFIG, EXCEPTIONS, HTMLREPORT, AWSRULESET, AWSCONFIG_FILE, EXCEPTIONS_FILE, HTMLREPORT_FILE, \
    GENERATOR_FILE, REPORT_TITLE
from ScoutSuite.output.js import JavaScriptReaderWriter
from ScoutSuite.output.utils import get_filename, prompt_4_overwrite


class HTMLReport(object):
    """
    Base HTML report
    """

    def __init__(self, profile, report_dir, timestamp=False, exceptions=None):
        exceptions = {} if exceptions is None else exceptions
        self.report_dir = report_dir
        self.profile = profile.replace('/', '_').replace('\\', '_')  # Issue 111
        self.current_time = datetime.datetime.now(dateutil.tz.tzlocal())
        if timestamp != False:
            self.timestamp = self.current_time.strftime("%Y-%m-%d_%Hh%M%z") if not timestamp else timestamp
            self.profile = '%s-%s' % (self.profile, self.timestamp)
        self.exceptions = exceptions
        self.scout2_report_data_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data')
        self.html_data_path = os.path.join(self.scout2_report_data_path, 'html')
        self.jsrw = JavaScriptReaderWriter(self.profile, report_dir, timestamp)

    def get_content_from(self, templates_type):
        contents = ''
        template_dir = os.path.join(self.html_data_path, templates_type)
        template_files = [os.path.join(template_dir, f) for f in os.listdir(template_dir) if
                          os.path.isfile(os.path.join(template_dir, f))]
        for filename in template_files:
            try:
                with open('%s' % filename, 'rt') as f:
                    contents = contents + f.read()
            except Exception as e:
                printException('Error reading filename %s: %s' % (filename, e))
        return contents

    def prepare_html_report_dir(self):
        if not os.path.isdir(self.report_dir):
            os.makedirs(self.report_dir)
        run_results_dir = os.path.join(self.report_dir, 'scoutsuite-results')
        if not os.path.isdir(run_results_dir):
            os.makedirs(run_results_dir)
        # Copy static 3rd-party files
        archive = os.path.join(self.scout2_report_data_path, 'includes.zip')
        zip_ref = zipfile.ZipFile(archive)
        zip_ref.extractall(self.report_dir)
        zip_ref.close()
        # Copy static files
        inc_scout2_dir = os.path.join(self.report_dir, 'inc-scoutsuite')
        src_inc_scout2_dir = os.path.join(self.scout2_report_data_path, 'inc-scoutsuite')
        if os.path.isdir(inc_scout2_dir):
            shutil.rmtree(inc_scout2_dir)
        shutil.copytree(src_inc_scout2_dir, inc_scout2_dir)


class Scout2Report(HTMLReport):
    """
    Scout2 HTML report
    """

    def __init__(self, provider, profile=None, report_dir=None, timestamp=False, exceptions=None):
        exceptions = {} if exceptions is None else exceptions
        self.html_root = HTMLREPORT_FILE
        self.provider = provider
        super(Scout2Report, self).__init__(profile, report_dir, timestamp, exceptions)

    def save(self, config, exceptions, force_write=False, debug=False):
        self.prepare_html_report_dir()
        self.jsrw.save_to_file(config, AWSCONFIG, force_write, debug)
        self.jsrw.save_to_file(exceptions, EXCEPTIONS, force_write, debug)
        return self.create_html_report(force_write)

    def create_html_report(self, force_write):
        contents = ''
        # Use all scripts under html/partials/
        contents += self.get_content_from('partials')
        contents += self.get_content_from('partials/%s' % self.provider)
        # Use all scripts under html/summaries/
        contents += self.get_content_from('summaries')
        contents += self.get_content_from('summaries/%s' % self.provider)
        new_file, first_line = get_filename(HTMLREPORT, self.profile, self.report_dir)
        printInfo('Creating %s ...' % new_file)
        if prompt_4_overwrite(new_file, force_write):
            if os.path.exists(new_file):
                os.remove(new_file)
            with open(os.path.join(self.html_data_path, self.html_root)) as f:
                with open(new_file, 'wt') as nf:
                    for line in f:
                        newline = line
                        if self.profile != 'default':
                            newline = newline.replace(AWSCONFIG_FILE,
                                                      AWSCONFIG_FILE.replace('.js', '-%s.js' % self.profile))
                            newline = newline.replace(EXCEPTIONS_FILE,
                                                      EXCEPTIONS_FILE.replace('.js', '-%s.js' % self.profile))
                        newline = newline.replace('<!-- PLACEHOLDER -->', contents)
                        nf.write(newline)
        return new_file


class RulesetGenerator(HTMLReport):
    """
    HTML ruleset generator for Scout2
    """

    def __init__(self, ruleset_name, report_dir=None, timestamp=False, exceptions=None):
        exceptions = {} if exceptions is None else exceptions
        self.html_root = GENERATOR_FILE
        self.ruleset_name = ruleset_name
        super(RulesetGenerator, self).__init__(ruleset_name, report_dir, timestamp, exceptions)

    def create_html_report(self, force_write):
        src_rule_generator = os.path.join(self.html_data_path, GENERATOR_FILE)
        rule_generator = os.path.join(self.report_dir, GENERATOR_FILE)
        shutil.copyfile(src_rule_generator, rule_generator)
        return rule_generator

    def save(self, config, force_write=False, debug=False):
        self.prepare_html_report_dir()
        self.jsrw.save_to_file(config, AWSRULESET, force_write, debug)
        return self.create_html_report(force_write)



# -*- coding: utf-8 -*-
"""
Base classes and functions for region-specific services
"""

import copy
import re
from threading import Thread
# Python2 vs Python3
try:
    from Queue import Queue
except ImportError:
    from queue import Queue

from opinel.utils.aws import build_region_list, connect_service, get_aws_account_id, get_name, handle_truncated_response
from opinel.utils.console import printException, printInfo
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.base.configs import resource_id_map
from ScoutSuite.providers.base.configs.threads import thread_configs
from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.utils import format_service_name, is_throttled
from ScoutSuite.providers.aws.configs.base import BaseConfig
from ScoutSuite.output.console import FetchStatusLogger

########################################
# Globals
########################################

api_clients = dict()

first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_caps_re = re.compile('([a-z0-9])([A-Z])')

########################################
# RegionalServiceConfig
########################################


class RegionalServiceConfig(object):
    """
    Single service configuration for non-global services

    :ivar regions:                      Dictionary of regions
    :ivar service:                      Name of the service
    """

    def __init__(self, service_metadata=None, thread_config=4):
        service_metadata = {} if service_metadata is None else service_metadata

        self.regions = {}
        self.thread_config = thread_configs[thread_config]
        self.service = \
            type(self).__name__.replace('Config', '').lower()  # TODO: use regex with EOS instead of plain replace

        # Booleans that define if threads should keep running
        self.run_q_threads = True
        self.run_qr_threads = True

        if service_metadata != {}:
            self.resource_types = {'global': [], 'region': [], 'vpc': []}
            self.targets = {'first_region': (), 'other_regions': ()}
            for resource in service_metadata['resources']:
                only_first_region = False
                if re.match(r'.*?\.vpcs\.id\..*?', service_metadata['resources'][resource]['path']):
                    self.resource_types['vpc'].append(resource)
                elif re.match(r'.*?\.regions\.id\..*?', service_metadata['resources'][resource]['path']):
                    self.resource_types['region'].append(resource)
                else:
                    only_first_region = True
                    self.resource_types['global'].append(resource)
                resource_metadata = service_metadata['resources'][resource]
                if 'api_call' not in resource_metadata:
                    continue
                params = resource_metadata['params'] if 'params' in resource_metadata else {}
                ignore_exceptions = True if 'no_exceptions' in resource_metadata and \
                                            resource_metadata['no_exceptions'] == True else False
                if not only_first_region:
                    self.targets['other_regions'] += ((resource,
                                                       resource_metadata['response'],
                                                       resource_metadata['api_call'],
                                                       params,
                                                       ignore_exceptions),)
                self.targets['first_region'] += ((resource,
                                                  resource_metadata['response'],
                                                  resource_metadata['api_call'],
                                                  params,
                                                  ignore_exceptions),)

    def init_region_config(self, region):
        """
        Initialize the region's configuration

        :param region:                  Name of the region
        """
        self.regions[region] = self.region_config_class(region_name = region, resource_types = self.resource_types)

    def fetch_all(self, credentials, regions = None, partition_name = 'aws', targets = None):
        """
        Fetch all the configuration supported by Scout2 for a given service

        :param credentials:             F
        :param service:                 Name of the service
        :param regions:                 Name of regions to fetch data from
        :param partition_name:          AWS partition to connect to
        :param targets:                 Type of resources to be fetched; defaults to all.

        """
        # Initialize targets
        # Tweak params
        regions = [] if regions is None else regions
        realtargets = ()
        if not targets:
            targets = self.targets
        for i, target in enumerate(targets['first_region']):
            params = self.tweak_params(target[3], credentials)
            realtargets = realtargets + ((target[0], target[1], target[2], params, target[4]),)
        targets['first_region'] = realtargets
        realtargets = ()
        for i, target in enumerate(targets['other_regions']):
            params = self.tweak_params(target[3], credentials)
            realtargets = realtargets + ((target[0], target[1], target[2], params, target[4]),)
        targets['other_regions'] = realtargets

        printInfo('Fetching %s config...' % format_service_name(self.service))
        self.fetchstatuslogger = FetchStatusLogger(targets['first_region'], True)
        api_service = 'ec2' if self.service.lower() == 'vpc' else self.service.lower()

        # Init regions
        regions = build_region_list(api_service, regions, partition_name) # TODO: move this code within this class
        self.fetchstatuslogger.counts['regions']['discovered'] = len(regions)

        # Threading to fetch & parse resources (queue consumer)
        q = self._init_threading(self._fetch_target, {}, self.thread_config['parse'])

        # Threading to list resources (queue feeder)
        qr = self._init_threading(self._fetch_region,
                                  {'api_service': api_service, 'credentials': credentials, 'q': q, 'targets': ()},
                                  self.thread_config['list'])

        # Go
        for i, region in enumerate(regions):
            qr.put((region, targets['first_region'] if i == 0 else targets['other_regions']))

        # Blocks until all items in the queue have been gotten and processed.
        qr.join()
        q.join()

        # Show completion and force newline
        self.fetchstatuslogger.show(True)

        # Threads should stop running as queues are empty
        self.run_qr_threads = False
        self.run_q_threads = False
        # Put x items in the queues to ensure threads run one last time (and exit)
        for i in range(self.thread_config['parse']):
            q.put(None)
        for j in range(self.thread_config['list']):
            qr.put(None)

    def _init_threading(self, function, params=None, num_threads=10):
        """
        Initialize queue and threads

        :param function:
        :param params:
        :param num_threads:
        :return:
        """
        params = {} if params is None else params
        q = Queue(maxsize=0) # TODO: find something appropriate
        for i in range(num_threads):
            worker = Thread(target=function, args=(q, params))
            worker.setDaemon(True)
            worker.start()
        return q

    def _fetch_region(self, q, params):
        global api_clients
        try:
            while self.run_qr_threads:
                try:
                    region, targets = q.get() or (None, None)
                    if region:
                        self.init_region_config(region)
                        api_client = connect_service(params['api_service'], params['credentials'], region, silent = True)
                        api_clients[region] = api_client
                        # TODO : something here for single_region stuff
                        self.regions[region].fetch_all(api_client, self.fetchstatuslogger, params['q'], targets)  # params['targets'])
                        self.fetchstatuslogger.counts['regions']['fetched'] += 1
                except Exception as e:
                    printException(e)
                finally:
                    q.task_done()
        except Exception as e:
            printException(e)
            pass

    def _fetch_target(self, q, params):
        try:
            while self.run_q_threads:
                try:
                    method, region, target = q.get() or (None, None, None)
                    if method:
                        backup = copy.deepcopy(target)

                        if method.__name__ == 'store_target':
                            target_type = target['scout2_target_type']
                        else:
                            target_type = method.__name__.replace('parse_', '') + 's'
                        method(params, region, target)
                        self.fetchstatuslogger.counts[target_type]['fetched'] += 1
                        self.fetchstatuslogger.show()
                except Exception as e:
                    if is_throttled(e):
                        q.put((method, region, backup))
                    else:
                        printException(e)
                finally:
                    q.task_done()
        except Exception as e:
            printException(e)
            pass

    def finalize(self):
        for t in self.fetchstatuslogger.counts:
            setattr(self, '%s_count' % t, self.fetchstatuslogger.counts[t]['fetched'])
        delattr(self, 'fetchstatuslogger')
        for r in self.regions:
            if hasattr(self.regions[r], 'fetchstatuslogger'):
                delattr(self.regions[r], 'fetchstatuslogger')

    def tweak_params(self, params, credentials):
        if type(params) == dict:
            for k in params:
                params[k] = self.tweak_params(params[k], credentials)
        elif type(params) == list:
            newparams = []
            for v in params:
                newparams.append(self.tweak_params(v, credentials))
            params = newparams
        else:
            if params == '_AWS_ACCOUNT_ID_':
                params = get_aws_account_id(credentials)
        return params

########################################
# RegionConfig
########################################


class RegionConfig(BaseConfig):
    """
    Base class for ...
    """

    def __init__(self, region_name, resource_types=None):
        resource_types = {} if resource_types is None else resource_types
        self.region = region_name
        self.name = region_name
        self.id = region_name
        for resource_type in resource_types['region'] + resource_types['global']:
            setattr(self, resource_type, {})
            setattr(self, '%s_count' % resource_type, 0)
        if len(resource_types['vpc']) > 0:
            setattr(self, 'vpcs', {})
            self.vpc_resource_types = resource_types['vpc']

    def fetch_all(self, api_client, fetchstatuslogger, q, targets):
        '''
        Make all API calls as defined in metadata.json

        :param api_client:
        :param fetchstatuslogger:
        :param q:
        :param targets:
        :return:
        '''
        self.fetchstatuslogger = fetchstatuslogger
        if targets != None:
            # Ensure targets is a tuple
            if type(targets) != list and type(targets) != tuple:
                targets = tuple(targets,)
            elif type(targets) != tuple:
                targets = tuple(targets)
        for target in targets:
            self._fetch_targets(api_client, q, target)

    def _fetch_targets(self, api_client, q, target):
        '''
        Make an API call defined in metadata.json.
        Parse the returned object as implemented in the "parse_[object name]" method.

        :param api_client:
        :param q:
        :param target:
        :return:
        '''
        # Handle & format the target type
        target_type, response_attribute, list_method_name, list_params, ignore_list_error = target
        list_method = getattr(api_client, list_method_name)
        try:
            targets = handle_truncated_response(list_method, list_params, [response_attribute])[response_attribute]
        except Exception as e:
            if not ignore_list_error:
                printException(e)
            targets = []
        setattr(self, '%s_count' % target_type, len(targets))
        self.fetchstatuslogger.counts[target_type]['discovered'] += len(targets)
        region = api_client._client_config.region_name
        # Queue resources
        for target in targets:
            # call callback methods
            try:
                callback = getattr(self, 'parse_%s' % target_type[0:-1])
            except:
                callback = self.store_target
                target['scout2_target_type'] = target_type
            if q:
                # Add to the queue
                q.put((callback, region, target))

    def store_target(self, global_params, region, target):
        target_type = target.pop('scout2_target_type')
        if 'VpcId' in target:
            vpc_id = target.pop('VpcId')
            manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
            tmp = getattr(self, 'vpcs')[vpc_id]
            target_dict = getattr(tmp, target_type)
        else:
            target_dict = getattr(self, target_type)
        target_id = target[resource_id_map[target_type]]
        get_name(target, target, resource_id_map[target_type])
        target_dict[target_id] = target




# -*- coding: utf-8 -*-

from opinel.utils.console import printError

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig


class StackdriverLoggingConfig(GCPBaseConfig):
    targets = (
        ('sinks', 'Sinks', 'list_sinks', {'project': '{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):
        self.library_type = 'cloud_client_library'

        self.sinks = {}
        self.sinks_count = 0

        super(StackdriverLoggingConfig, self).__init__(thread_config)

    def parse_sinks(self, sink, params):

        sink_dict = {}
        sink_dict['name'] = sink.name
        sink_dict['filter'] = sink.filter_
        sink_dict['destination'] = sink.destination

        self.sinks[sink_dict['name']] = sink_dict



# -*- coding: utf-8 -*-

from opinel.utils.console import printError, printException, printDebug
from opinel.utils.aws import get_aws_account_id, get_partition_name

class BaseServicesConfig(object):

    def __init__(self, metadata=None, thread_config=4):
        pass

    def _is_provider(self, provider_name):
        return False

    def fetch(self, credentials, services=None, regions=None):
        services = [] if services is None else services
        regions = [] if regions is None else regions
        for service in vars(self):
            try:
                # skip services
                if services != [] and service not in services:
                    continue
                service_config = getattr(self, service)
                # call fetch method for the service
                if 'fetch_all' in dir(service_config):
                    method_args = {}
                    method_args['credentials'] = credentials
                    method_args['regions'] = regions

                    if self._is_provider('aws'):
                        if service != 'iam':
                            method_args['partition_name'] = get_partition_name(credentials)

                    service_config.fetch_all(**method_args)
                    if hasattr(service_config, 'finalize'):
                        service_config.finalize()
                else:
                    printDebug('No method to fetch service %s.' % service)
            except Exception as e:
                printError('Error: could not fetch %s configuration.' % service)
                printException(e)

    # TODO is this ever called?
    # def postprocessing(self):
    #     pass



# -*- coding: utf-8 -*-
from __future__ import print_function

import re



########################################
# Globals
########################################

ec2_classic = 'EC2-Classic'

formatted_service_name = {
    'cloudformation': 'CloudFormation',
    'cloudtrail': 'CloudTrail',
    'cloudwatch': 'CloudWatch',
    'directconnect': 'Direct Connect',
    'elasticache': 'ElastiCache',
    'lambda': 'Lambda',
    'redshift': 'RedShift',
    'route53': 'Route53',
    'route53domains': 'Route53Domains',

    'cloudstorage': 'Cloud Storage',
    'cloudsql': 'Cloud SQL',
    'stackdriverlogging': 'Stackdriver Logging',
    'stackdrivermonitoring': 'Stackdriver Monitoring',
    'computeengine': 'Compute Engine',
    'kubernetesengine': 'Kubernetes Engine',
    'cloudresourcemanager': 'Cloud Resource Manager',

    'monitor': 'Monitor',
    'storageaccounts': 'Storage Accounts',
    'sqldatabase': 'SQL Database',
    'securitycenter': 'Security Center',
    'keyvault': 'Key Vault',
    'appgateway': 'Application Gateway',
    'rediscache': 'Redis Cache',
    'network': 'Network',
    'appservice': 'App Service',
    'loadbalancer': 'Load Balancer'
}


########################################
# Functions
########################################

def format_service_name(service):
    """

    :param service:
    :return:
    """
    return formatted_service_name[service] if service in formatted_service_name else service.upper()


def get_keys(src, dst, keys):
    """
    Copies the value of keys from source object to dest object

    :param src:
    :param dst:
    :param keys:
    :return:
    """
    for key in keys:
        #dst[no_camel(key)] = src[key] if key in src else None
        dst[key] = src[key] if key in src else None


def no_camel(name):
    """
    Converts CamelCase to camel_case

    :param name:
    :return:
    """
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def is_throttled(e):
    """
    Determines whether the exception is due to API throttling.

    :param e:                           Exception raised
    :return:                            True if it's a throttling exception else False
    """
    return True if  (hasattr(e, 'response') and 'Error' in e.response and e.response['Error']['Code'] in [ 'Throttling', 'RequestLimitExceeded', 'ThrottlingException' ]) else False



# -*- coding: utf-8 -*-

from opinel.utils.console import printException

from google.cloud import storage
from google.cloud import logging as stackdriver_logging
from google.cloud import monitoring_v3
from google.cloud import container_v1

from googleapiclient import discovery

import logging

def gcp_connect_service(service, credentials=None, region_name=None):

    logging.getLogger('googleapiclient.discovery_cache').setLevel(logging.ERROR)
    # Set logging level to error for GCP services as otherwise generates a lot of warnings
    logging.getLogger().setLevel(logging.ERROR)

    try:

        if service == 'cloudresourcemanager':
            return discovery.build('cloudresourcemanager', 'v1', cache_discovery=False, cache=MemoryCache())

        elif service == 'cloudresourcemanager-v2':
            return discovery.build('cloudresourcemanager', 'v2', cache_discovery=False, cache=MemoryCache())

        elif service == 'cloudstorage':
            # return storage.Client()
            return storage.Client(project='placeholder')  # need a project value to instantiate the client event though
                                                          # it won't be used afterwards

        elif service == 'cloudsql':
            return discovery.build('sqladmin', 'v1beta4', cache_discovery=False, cache=MemoryCache())

        elif service == 'iam':
            return discovery.build('iam', 'v1', cache_discovery=False, cache=MemoryCache())

        if service == 'stackdriverlogging':
            # return stackdriver_logging.LoggingServiceV2Client()
            return stackdriver_logging.Client(project='placeholder')  # need a project value to instantiate the client
                                                                      # event though it won't be used afterwards

        if service == 'stackdrivermonitoring':
            return monitoring_v3.MetricServiceClient()

        elif service == 'computeengine':
            return discovery.build('compute', 'v1', cache_discovery=False, cache=MemoryCache())

        elif service == 'kubernetesengine':
            return container_v1.ClusterManagerClient()

        else:
            printException('Service %s not supported' % service)
            return None

    except Exception as e:
        printException(e)
        return None


class MemoryCache:
    """
    Workaround https://github.com/googleapis/google-api-python-client/issues/325#issuecomment-274349841
    """
    _CACHE = {}

    def get(self, url):
        return MemoryCache._CACHE.get(url)

    def set(self, url, content):
        MemoryCache._CACHE[url] = content


# -*- coding: utf-8 -*-

import json

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients



########################################
# SESRegionConfig
########################################

class SESRegionConfig(RegionConfig):
    """
    SES configuration for a single AWS region
    """

    def parse_identitie(self, global_params, region, identity_name):
        """
        Parse a single identity and fetch additional attributes

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        """
        identity = {'name': identity_name, 'policies': {}}
        policy_names = api_clients[region].list_identity_policies(Identity = identity_name)['PolicyNames']
        if len(policy_names):
            policies = api_clients[region].get_identity_policies(Identity = identity_name, PolicyNames = policy_names)['Policies']
            for policy_name in policies:
                identity['policies'][policy_name] = json.loads(policies[policy_name])
        dkim = api_clients[region].get_identity_dkim_attributes(Identities = [ identity_name ])['DkimAttributes'][identity_name]
        identity['DkimEnabled'] = dkim['DkimEnabled']
        identity['DkimVerificationStatus'] = dkim['DkimVerificationStatus']
        self.identities[self.get_non_provider_id(identity_name)] = identity



########################################
# SESConfig
########################################

class SESConfig(RegionalServiceConfig):
    """
    SES configuration for all AWS regions
    """

    region_config_class = SESRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(SESConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-[d['value'] for d in l]

import os
import warnings

import google.auth
import googleapiclient
from opinel.utils.console import printError, printException, printInfo

from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.providers.gcp.configs.services import GCPServicesConfig
from ScoutSuite.providers.gcp.utils import gcp_connect_service


class GCPCredentials():

    def __init__(self, api_client_credentials, cloud_client_credentials):
        self.api_client_credentials = api_client_credentials
        self.cloud_client_credentials = cloud_client_credentials


class GCPProvider(BaseProvider):
    """
    Implements provider for GCP
    """

    def __init__(self, project_id=None, folder_id=None, organization_id=None, all_projects=None,
                 report_dir=None, timestamp=None, services=None, skipped_services=None, thread_config=4, **kwargs):
        services = [] if services is None else services
        skipped_services = [] if skipped_services is None else skipped_services

        self.profile = 'gcp-profile'  # TODO this is aws-specific

        self.metadata_path = '%s/metadata.json' % os.path.split(os.path.abspath(__file__))[0]

        self.provider_code = 'gcp'
        self.provider_name = 'Google Cloud Platform'

        self.projects = []
        self.all_projects = all_projects
        self.project_id = project_id
        self.folder_id = folder_id
        self.organization_id = organization_id

        self.services_config = GCPServicesConfig

        super(GCPProvider, self).__init__(report_dir, timestamp, services, skipped_services, thread_config)

    def authenticate(self, user_account=None, service_account=None, **kargs):
        """
        Implement authentication for the GCP provider
        Refer to https://google-auth.readthedocs.io/en/stable/reference/google.auth.html.

        :return:
        """

        if user_account:
            # disable GCP warning about using User Accounts
            warnings.filterwarnings("ignore", "Your application has authenticated using end user credentials")
            pass  # Nothing more to do
        elif service_account:
            client_secrets_path = os.path.abspath(service_account)
            os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = client_secrets_path
        else:
            printError('Failed to authenticate to GCP - no supported account type')
            return False

        try:

            self.credentials, project_id = google.auth.default()
            if self.credentials:

                # All projects to which the user / Service Account has access to
                if self.all_projects:
                    self.projects = self._get_projects(parent_type='all',
                                                       parent_id=None)
                    if service_account and hasattr(self.credentials, 'service_account_email'):
                        self.aws_account_id = self.credentials.service_account_email  # FIXME this is for AWS
                    else:
                        self.aws_account_id = 'GCP'  # FIXME this is for AWS
                    self.profile = 'GCP'  # FIXME this is for AWS
                # Project passed through the CLI
                elif self.project_id:
                    self.projects = self._get_projects(parent_type='project',
                                                       parent_id=self.project_id)
                    self.aws_account_id = self.project_id  # FIXME this is for AWS
                    self.profile = self.project_id  # FIXME this is for AWS

                # Folder passed through the CLI
                elif self.folder_id:
                    self.projects = self._get_projects(parent_type='folder',
                                                       parent_id=self.folder_id)
                    self.aws_account_id = self.folder_id  # FIXME this is for AWS
                    self.profile = self.folder_id  # FIXME this is for AWS

                # Organization passed through the CLI
                elif self.organization_id:
                    self.projects = self._get_projects(parent_type='organization',
                                                       parent_id=self.organization_id)
                    self.aws_account_id = self.organization_id  # FIXME this is for AWS
                    self.profile = self.organization_id  # FIXME this is for AWS

                # Project inferred from default configuration
                elif project_id:
                    self.projects = self._get_projects(parent_type='project',
                                                       parent_id=project_id)
                    self.aws_account_id = project_id  # FIXME this is for AWS
                    self.profile = project_id  # FIXME this is for AWS

                # Raise exception if none of the above
                else:
                    printInfo("Could not infer the Projects to scan and no default Project ID was found.")
                    return False

                # TODO this shouldn't be done here? but it has to in order to init with projects...
                self.services.set_projects(projects=self.projects)

                return True
            else:
                return False

        except google.auth.exceptions.DefaultCredentialsError as e:
            printError('Failed to authenticate to GCP')
            printException(e)
            return False

        except googleapiclient.errors.HttpError as e:
            printError('Failed to authenticate to GCP')
            printException(e)
            return False

    def preprocessing(self, ip_ranges=None, ip_ranges_name_key=None):
        """
        TODO description
        Tweak the AWS config to match cross-resources and clean any fetching artifacts

        :param ip_ranges:
        :param ip_ranges_name_key:
        :return: None
        """
        ip_ranges = [] if ip_ranges is None else ip_ranges

        self._match_instances_and_snapshots()
        self._match_networks_and_instances()

        super(GCPProvider, self).preprocessing()

    def _get_projects(self, parent_type, parent_id):
        """
        Returns all the projects in a given organization or folder. For a project_id it only returns the project
        details.
        """

        if parent_type not in ['project', 'organization', 'folder', 'all']:
            return None

        projects = []

        # FIXME can't currently be done with API client library as it consumes v1 which doesn't support folders
        """

        resource_manager_client = resource_manager.Client(credentials=self.credentials)

        project_list = resource_manager_client.list_projects()

        for p in project_list:
            if p.parent['id'] == self.organization_id and p.status == 'ACTIVE':
                projects.append(p.project_id)
        """

        resource_manager_client_v1 = gcp_connect_service(service='cloudresourcemanager', credentials=self.credentials)
        resource_manager_client_v2 = gcp_connect_service(service='cloudresourcemanager-v2',
                                                         credentials=self.credentials)

        try:
            if parent_type == 'project':
                project_response = resource_manager_client_v1.projects().list(filter='id:%s' % parent_id).execute()
                if 'projects' in project_response.keys():
                    for project in project_response['projects']:
                        if project['lifecycleState'] == "ACTIVE":
                            projects.append(project)

            elif parent_type == 'all':
                project_response = resource_manager_client_v1.projects().list().execute()
                if 'projects' in project_response.keys():
                    for project in project_response['projects']:
                        if project['lifecycleState'] == "ACTIVE":
                            projects.append(project)
            else:

                # get parent children projects
                request = resource_manager_client_v1.projects().list(filter='parent.id:%s' % parent_id)
                while request is not None:
                    response = request.execute()

                    if 'projects' in response.keys():
                        for project in response['projects']:
                            if project['lifecycleState'] == "ACTIVE":
                                projects.append(project)

                    request = resource_manager_client_v1.projects().list_next(previous_request=request,
                                                                              previous_response=response)

                # get parent children projects in children folders recursively
                folder_response = resource_manager_client_v2.folders().list(
                    parent='%ss/%s' % (parent_type, parent_id)).execute()
                if 'folders' in folder_response.keys():
                    for folder in folder_response['folders']:
                        projects.extend(self._get_projects("folder", folder['name'].strip(u'folders/')))

            printInfo("Found {} project(s) to scan.".format(len(projects)))

        except Exception as e:
            printError('Unable to list accessible Projects')
            printException(e)

        finally:
            return projects


    def _match_instances_and_snapshots(self):
        """
        Compare Compute Engine instances and snapshots to identify instance disks that do not have a snapshot.

        :return:
        """

        if 'computeengine' in self.services:
            for instance in self.services['computeengine']['instances'].values():
                for instance_disk in instance['disks'].values():
                    instance_disk['snapshots'] = []
                    for disk in self.services['computeengine']['snapshots'].values():
                        if disk['status'] == 'READY' and disk['source_disk_url'] == instance_disk['source_url']:
                            instance_disk['snapshots'].append(disk)

                    instance_disk['latest_snapshot'] = max(instance_disk['snapshots'],
                                                           key=lambda x: x['creation_timestamp']) \
                        if instance_disk['snapshots'] else None

    def _match_networks_and_instances(self):
        """
        For each network, math instances in that network

        :return:
        """

        if 'computeengine' in self.services:
            for network in self.services['computeengine']['networks'].values():
                network['instances'] = []
                for instance in self.services['computeengine']['instances'].values():
                    for network_interface in instance['network_interfaces']:
                        if network_interface['network'] == network['network_url']:
                            network['instances'].append(instance['id'])



# -*- coding: utf-8 -*-

import os
import json

from getpass import getpass

from opinel.utils.console import printError, printException

from ScoutSuite.providers.base.provider import BaseProvider
from ScoutSuite.providers.azure.configs.services import AzureServicesConfig

from msrestazure.azure_active_directory import MSIAuthentication
from azure.mgmt.resource import SubscriptionClient
from azure.common.credentials import ServicePrincipalCredentials, UserPassCredentials, get_azure_cli_credentials


class AzureCredentials:

    def __init__(self, credentials, subscription_id):
        self.credentials = credentials
        self.subscription_id = subscription_id


class AzureProvider(BaseProvider):
    """
    Implements provider for Azure
    """

    def __init__(self, project_id=None, organization_id=None,
                 report_dir=None, timestamp=None, services=None, skipped_services=None, thread_config=4, **kwargs):
        services = [] if services is None else services
        skipped_services = [] if skipped_services is None else skipped_services

        self.profile = 'azure-profile'  # TODO this is aws-specific

        self.metadata_path = '%s/metadata.json' % os.path.split(os.path.abspath(__file__))[0]

        self.provider_code = 'azure'
        self.provider_name = 'Microsoft Azure'

        self.services_config = AzureServicesConfig

        super(AzureProvider, self).__init__(report_dir, timestamp, services, skipped_services, thread_config)

    def authenticate(self, cli=None, msi=None, service_principal=None, file_auth=None, user_account=None,
                     tenant_id=None, subscription_id=None, client_id=None, client_secret=None, username=None,
                     password=None, **kargs):
        """
        Implements authentication for the Azure provider using azure-cli.
        Refer to https://docs.microsoft.com/en-us/python/azure/python-sdk-azure-authenticate?view=azure-python.

        :return:
        """

        try:
            if cli:
                cli_credentials, self.aws_account_id = get_azure_cli_credentials()  # TODO: Remove aws_account_id
                self.credentials = AzureCredentials(cli_credentials, self.aws_account_id)
                return True
            elif msi:
                credentials = MSIAuthentication()

                # Get the subscription ID
                subscription_client = SubscriptionClient(credentials)
                try:
                    # Tries to read the subscription list
                    subscription = next(subscription_client.subscriptions.list())
                    self.aws_account_id = subscription.subscription_id
                except StopIteration:
                    # If the VM cannot read subscription list, ask Subscription ID:
                    self.aws_account_id = input('Subscription ID: ')

                self.credentials = AzureCredentials(credentials, self.aws_account_id)
                return True
            elif file_auth:
                data = json.loads(file_auth.read())
                subscription_id = data.get('subscriptionId')
                tenant_id = data.get('tenantId')
                client_id = data.get('clientId')
                client_secret = data.get('clientSecret')

                self.aws_account_id = tenant_id  # TODO this is for AWS

                credentials = ServicePrincipalCredentials(
                    client_id=client_id,
                    secret=client_secret,
                    tenant=tenant_id
                )

                self.credentials = AzureCredentials(credentials, subscription_id)

                return True
            elif service_principal:
                subscription_id = subscription_id if subscription_id else input("Subscription ID: ")
                tenant_id = tenant_id if tenant_id else input("Tenant ID: ")
                client_id = client_id if client_id else input("Client ID: ")
                client_secret = client_secret if client_secret else getpass("Client secret: ")

                self.aws_account_id = tenant_id  # TODO this is for AWS

                credentials = ServicePrincipalCredentials(
                    client_id=client_id,
                    secret=client_secret,
                    tenant=tenant_id
                )

                self.credentials = AzureCredentials(credentials, subscription_id)

                return True
            elif user_account:
                username = username if username else input("Username: ")
                password = password if password else getpass("Password: ")

                credentials = UserPassCredentials(username, password)

                if subscription_id:
                    self.aws_account_id = subscription_id
                else:
                    # Get the subscription ID
                    subscription_client = SubscriptionClient(credentials)
                    try:
                        # Tries to read the subscription list
                        subscription = next(subscription_client.subscriptions.list())
                        self.aws_account_id = subscription.subscription_id
                    except StopIteration:
                        # If the user cannot read subscription list, ask Subscription ID:
                        self.aws_account_id = input('Subscription ID: ')

                self.credentials = AzureCredentials(credentials, self.aws_account_id)
                return True
        except Exception as e:
            printError('Failed to authenticate to Azure')
            printException(e)
            return False

    def preprocessing(self, ip_ranges=None, ip_ranges_name_key=None):
        """
        TODO description
        Tweak the AWS config to match cross-service resources and clean any fetching artifacts

        :param ip_ranges:
        :param ip_ranges_name_key:
        :return: None
        """
        ip_ranges = [] if ip_ranges is None else ip_ranges
        super(AzureProvider, self).preprocessing()



# -*- coding: utf-8 -*-

from ScoutSuite.providers.base.configs.services import BaseServicesConfig
from ScoutSuite.providers.azure.services.storageaccounts import StorageAccountsConfig
from ScoutSuite.providers.azure.services.monitor import MonitorConfig
from ScoutSuite.providers.azure.services.sqldatabase import SQLDatabaseConfig
from ScoutSuite.providers.azure.services.securitycenter import SecurityCenterConfig
from ScoutSuite.providers.azure.services.network import NetworkConfig
from ScoutSuite.providers.azure.services.keyvault import KeyVaultConfig
try:
    from ScoutSuite.providers.azure.services.appgateway_private import AppGatewayConfig
except ImportError:
    pass
try:
    from ScoutSuite.providers.azure.services.rediscache_private import RedisCacheConfig
except ImportError:
    pass
try:
    from ScoutSuite.providers.azure.services.appservice_private import AppServiceConfig
except ImportError:
    pass
try:
    from ScoutSuite.providers.azure.services.loadbalancer_private import LoadBalancerConfig
except ImportError:
    pass


class AzureServicesConfig(BaseServicesConfig):

    def __init__(self, metadata=None, thread_config=4, **kwargs):

        self.storageaccounts = StorageAccountsConfig(thread_config=thread_config)
        self.monitor = MonitorConfig(thread_config=thread_config)
        self.sqldatabase = SQLDatabaseConfig(thread_config=thread_config)
        self.securitycenter = SecurityCenterConfig(thread_config=thread_config)
        self.network = NetworkConfig(thread_config=thread_config)
        self.keyvault = KeyVaultConfig(thread_config=thread_config)

        try:
            self.appgateway = AppGatewayConfig(thread_config=thread_config)
        except NameError:
            pass
        try:
            self.rediscache = RedisCacheConfig(thread_config=thread_config)
        except NameError:
            pass
        try:
            self.appservice = AppServiceConfig(thread_config=thread_config)
        except NameError:
            pass
        try:
            self.loadbalancer = LoadBalancerConfig(thread_config=thread_config)
        except NameError:
            pass

    def _is_provider(self, provider_name):
        return provider_name == 'azure'



# -*- coding: utf-8 -*-
"""
Exceptions handling
"""

from opinel.utils.console import printDebug

from ScoutSuite import EXCEPTIONS
from ScoutSuite.output.js import JavaScriptReaderWriter


class RuleExceptions(object):

    def __init__(self, profile, file_path=None, foobar=None):
        self.profile = profile
        self.file_path = file_path
        self.jsrw = JavaScriptReaderWriter(self.profile)
        self.exceptions = self.jsrw.load_from_file(config_type=EXCEPTIONS,
                                                   config_path=self.file_path,
                                                   first_line=True)

    def process(self, cloud_provider):
        for service in self.exceptions:
            for rule in self.exceptions[service]:
                filtered_items = []
                if rule not in cloud_provider.services[service]['findings']:
                    printDebug('Warning:: key error should not be happening')
                    continue
                for item in cloud_provider.services[service]['findings'][rule]['items']:
                    if item not in self.exceptions[service][rule]:
                        filtered_items.append(item)
                cloud_provider.services[service]['findings'][rule]['items'] = filtered_items
                cloud_provider.services[service]['findings'][rule]['flagged_items'] = \
                    len(cloud_provider.services[service]['findings'][rule]['items'])



# -*- coding: utf-8 -*-

from opinel.utils.console import printError

from ScoutSuite.providers.gcp.configs.base import GCPBaseConfig


class CloudStorageConfig(GCPBaseConfig):
    targets = (
        ('buckets', 'Buckets', 'list_buckets', {'project': '{{project_placeholder}}'}, False),
    )

    def __init__(self, thread_config):
        self.library_type = 'cloud_client_library'

        self.buckets = {}
        self.buckets_count = 0

        super(CloudStorageConfig, self).__init__(thread_config)

    def parse_buckets(self, bucket, params):
        """
        Parse a single Cloud Storage bucket

        :param bucket: Bucket object  representing a single bucket
        :param params: Dictionary of parameters, including API client
        :return: None
        """
        api_client = params['api_client']

        bucket._client = api_client

        bucket_dict = {}
        bucket_dict['id'] = self.get_non_provider_id(bucket.id)
        bucket_dict['name'] = bucket.name

        for project in self.projects:
            if str(project['projectNumber']) == str(bucket.project_number):
                bucket_dict['project_id'] = project['projectId']
                break

        bucket_dict['project_number'] = bucket.project_number
        bucket_dict['creation_date'] = bucket.time_created
        bucket_dict['location'] = bucket.location
        bucket_dict['storage_class'] = bucket.storage_class.lower()
        bucket_dict['versioning_status_enabled'] = bucket.versioning_enabled

        get_cloudstorage_bucket_logging(bucket, bucket_dict)
        get_cloudstorage_bucket_acl(bucket, bucket_dict)

        self.buckets[bucket_dict['id']] = bucket_dict


def get_cloudstorage_bucket_logging(bucket, bucket_dict):
    try:
        bucket_dict['logging_enabled'] = bucket.get_logging() is not None
        return True
    except Exception as e:
        printError('Failed to get bucket logging configuration for %s: %s' % (bucket.name, e))
        bucket_dict['logging_enabled'] = None
        return False


def get_cloudstorage_bucket_acl(bucket, bucket_dict):
    try:

        bucket_acls = bucket.get_iam_policy()

        bucket_dict['acl_configuration'] = {}
        for role in bucket_acls._bindings:
            for member in bucket_acls[role]:
                if member.split(':')[0] not in ['projectEditor', 'projectViewer', 'projectOwner']:
                    if member not in bucket_dict['acl_configuration']:
                        bucket_dict['acl_configuration'][member] = [role]
                    else:
                        bucket_dict['acl_configuration'][member].append(role)

        return True
    except Exception as e:
        printError('Failed to get bucket ACL configuration for %s: %s' % (bucket.name, e))
        bucket_dict['acls'] = 'Unknown'
        return False



# -*- coding: utf-8 -*-
"""
ELBv2-related classes and functions
"""

from opinel.utils.aws import handle_truncated_response
from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients
from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.utils import ec2_classic


########################################
# ELBv2RegionConfig
########################################

class ELBv2RegionConfig(RegionConfig):
    """
    ELBv2 configuration for a single AWS region

    :ivar vpcs:                         Dictionary of VPCs [id]
    """

    def parse_lb(self, global_params, region, lb):
        """

        :param global_params:
        :param region:
        :param source:
        :return:
        """
        lb['arn'] = lb.pop('LoadBalancerArn')
        lb['name'] = lb.pop('LoadBalancerName')
        vpc_id = lb.pop('VpcId') if 'VpcId' in lb and lb['VpcId'] else ec2_classic
        manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
        lb['security_groups'] = []
        try:
            for sg in lb['SecurityGroups']:
                lb['security_groups'].append({'GroupId': sg})
            lb.pop('SecurityGroups')
        except Exception as e:
            # Network load balancers do not have security groups
            pass
        lb['listeners'] = {}
        # Get listeners
        listeners = handle_truncated_response(api_clients[region].describe_listeners, {'LoadBalancerArn': lb['arn']}, ['Listeners'])['Listeners']
        for listener in listeners:
            listener.pop('ListenerArn')
            listener.pop('LoadBalancerArn')
            port = listener.pop('Port')
            lb['listeners'][port] = listener
        # Get attributes
        lb['attributes'] = api_clients[region].describe_load_balancer_attributes(LoadBalancerArn = lb['arn'])['Attributes']
        self.vpcs[vpc_id].lbs[self.get_non_provider_id(lb['name'])] = lb

    def parse_ssl_policie(self, global_params, region, policy):
        id = self.get_non_provider_id(policy['Name'])
        self.ssl_policies[id] = policy


########################################
# ELBv2Config
########################################

class ELBv2Config(RegionalServiceConfig):
    """
    ELBv2 configuration for all AWS regions
    """
    region_config_class = ELBv2RegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(ELBv2Config, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

__author__ = 'NCC Group'
__version__ = '4.2.post2'

AWSCONFIG  = 42
EXCEPTIONS = 4242
HTMLREPORT = 424242
AWSRULESET = 42424242

DEFAULT_REPORT_DIR = 'scoutsuite-report'
AWSCONFIG_FILE = 'scoutsuite-results/scoutsuite_results.js'
AWSRULESET_FILE = 'scoutsuite-results/scoutsuite_ruleset.js'
EXCEPTIONS_FILE = 'scoutsuite-results/scoutsuite_exceptions.js'
HTMLREPORT_FILE = 'report.html'
GENERATOR_FILE = 'ruleset-generator.html'

REPORT_TITLE = 'Scout Suite Report'



# -*- coding: utf-8 -*-
"""
Lambda-related classes and functions
"""

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients



########################################
# LambdaRegionConfig
########################################

class LambdaRegionConfig(RegionConfig):

    def parse_function(self, global_params, region, function):
        function['name'] = function.pop('FunctionName')
        self.functions[function['name']] = function



########################################
# LambdaConfig
########################################

class LambdaConfig(RegionalServiceConfig):
    """
    Lambda configuration for all AWS regions
    """

    region_config_class = LambdaRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(LambdaConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

from opinel.utils.globals import manage_dictionary

from ScoutSuite.providers.aws.configs.regions import RegionalServiceConfig, RegionConfig, api_clients
from ScoutSuite.providers.aws.configs.vpc import VPCConfig
from ScoutSuite.utils import ec2_classic



########################################
# ElastiCacheRegionConfig
########################################

class ElastiCacheRegionConfig(RegionConfig):
    """
    ElastiCache configuration for a single AWS region
    """

    def parse_cluster(self, global_params, region, cluster):
        """
        Parse a single ElastiCache cluster

        :param global_params:           Parameters shared for all regions
        :param region:                  Name of the AWS region
        :param cluster:                 ElastiCache cluster
        """
        cluster_name = cluster.pop('CacheClusterId')
        cluster['name'] = cluster_name
        # Must fetch info about the subnet group to retrieve the VPC ID...
        if 'CacheSubnetGroupName' in cluster:
            subnet_group = api_clients[region].describe_cache_subnet_groups(CacheSubnetGroupName = cluster['CacheSubnetGroupName'])['CacheSubnetGroups'][0]
            vpc_id = subnet_group['VpcId']
        else:
            vpc_id = ec2_classic
            subnet_group = None
        manage_dictionary(self.vpcs, vpc_id, VPCConfig(self.vpc_resource_types))
        self.vpcs[vpc_id].clusters[cluster_name] = cluster
        if subnet_group:
            self.vpcs[vpc_id].subnet_groups[subnet_group['CacheSubnetGroupName']] = subnet_group


    def parse_security_group(self, global_params, region, security_group):
        """
        Parse a single ElastiCache security group

        :param global_params:
        :param region:
        :param security_group:
        :return:
        """
        security_group['name'] = security_group.pop('CacheSecurityGroupName')
        self.security_groups[security_group['name']] = security_group



########################################
# ElastiCacheConfig
########################################

class ElastiCacheConfig(RegionalServiceConfig):
    """
    ElastiCache configuration for all AWS regions
    """

    region_config_class = ElastiCacheRegionConfig

    def __init__(self, service_metadata, thread_config = 4):
        super(ElastiCacheConfig, self).__init__(service_metadata, thread_config)



# -*- coding: utf-8 -*-

from ScoutSuite.providers.azure.configs.base import AzureBaseConfig


class SecurityCenterConfig(AzureBaseConfig):
    targets = (
        ('pricings', 'Pricings', 'list', {}, False),
        ('security_contacts', 'Security Contacts', 'list', {}, False),
        ('auto_provisioning_settings', 'Auto Provisioning Settings', 'list', {}, False)
    )

    def __init__(self, thread_config):
        self.pricings = {}
        self.pricings_count = 0

        self.security_contacts = {}
        self.security_contacts_count = 0

        self.auto_provisioning_settings = {}
        self.auto_provisioning_settings_count = 0

        super(SecurityCenterConfig, self).__init__(thread_config)

    def parse_pricings(self, pricing, params):
        pricing_dict = {}
        pricing_dict['id'] = pricing.id
        pricing_dict['name'] = pricing.name
        pricing_dict['pricing_tier'] = pricing.pricing_tier

        self.pricings[pricing_dict['id']] = pricing_dict

    def parse_security_contacts(self, security_contact, params):
        security_contact_dict = {}
        security_contact_dict['id'] = security_contact.id
        security_contact_dict['name'] = security_contact.name
        security_contact_dict['email'] = security_contact.email
        security_contact_dict['phone'] = security_contact.phone
        security_contact_dict['alert_notifications'] = security_contact.alert_notifications == "On"
        security_contact_dict['alerts_to_admins'] = security_contact.alerts_to_admins == "On"
        security_contact_dict['additional_properties'] = security_contact.additional_properties

        self.security_contacts[security_contact_dict['id']] = security_contact_dict

    def parse_auto_provisioning_settings(self, auto_provisioning_setting, params):
        auto_provisioning_setting_dict = {}
        auto_provisioning_setting_dict['id'] = auto_provisioning_setting.id
        auto_provisioning_setting_dict['name'] = auto_provisioning_setting.name
        auto_provisioning_setting_dict['auto_provision'] = auto_provisioning_setting.auto_provision

        self.auto_provisioning_settings[auto_provisioning_setting_dict['id']] = auto_provisioning_setting_dict



#!/usr/bin/env python

# distutils/setuptools install script for Scout Suite
import os
from setuptools import setup, find_packages

# Package info
NAME = 'ScoutSuite'
ROOT = os.path.dirname(__file__)
VERSION = __import__(NAME).__version__

# Requirements
requirements = []
with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'requirements.txt')) as f:
    for r in f.readlines():
        requirements.append(r.strip())

# Setup
setup(
    name=NAME,
    version=VERSION,
    description='Scout Suite, a multi-cloud security auditing tool',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    author='NCC Group',
    url='https://github.com/nccgroup/ScoutSuite',
    entry_points={
        'console_scripts': [
            'Scout = ScoutSuite.__main__:main',
            'Scout2RulesGenerator = ScoutSuite.__rules_generator__:main',
            'Scout2Listall = ScoutSuite.__listall__:main'
        ]
    },
    packages=find_packages(),
    package_data={
        'ScoutSuite.output': [
            'data/html/*.html',
            'data/html/partials/*.html',
            'data/html/partials/aws/*.html',
            'data/html/partials/azure/*.html',
            'data/html/partials/gcp/*.html',
            'data/html/summaries/*.html',
            'data/includes.zip',
            'data/inc-scout2/*.js',
            'data/inc-scout2/*.css'
        ],
        'ScoutSuite.providers': [
            'aws/rules/conditions/*.json',
            'aws/rules/filters/*.json',
            'aws/rules/findings/*.json',
            'aws/rules/rulesets/*.json'
            'azure/rules/conditions/*.json',
            'azure/rules/filters/*.json',
            'azure/rules/findings/*.json',
            'azure/rules/rulesets/*.json'
            'gcp/rules/conditions/*.json',
            'gcp/rules/filters/*.json',
            'gcp/rules/findings/*.json',
            'gcp/rules/rulesets/*.json'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license='GNU General Public License v2 (GPLv2)',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
)



# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import unicode_literals

import json
import sys
import copy

from opinel.utils.console import printException, printInfo
from opinel.utils.globals import manage_dictionary

from ScoutSuite import __version__ as scout2_version
from ScoutSuite.providers.base.configs.browser import get_object_at
from ScoutSuite.output.html import Scout2Report


class BaseProvider(object):
    """
    Base class for the different providers.

    Root object that holds all of the necessary provider-specific resources and Scout configuration items.

    For each supported cloud provider, a child class will be created which implements the necessary code.
    In addition, each method of children classes will call the base provider in order to execute code required for
    all cloud providers
    """

    def __init__(self, report_dir=None, timestamp=None, services=None, skipped_services=None, thread_config=4, **kwargs):
        """

        :aws_account_id     AWS account ID
        :last_run           Information about the last run
        :metadata           Metadata used to generate the HTML report
        :ruleset            Ruleset used to perform the analysis
        :services           AWS configuration sorted by service
        """
        services = [] if services is None else services
        skipped_services = [] if skipped_services is None else skipped_services

        self.credentials = None
        self.last_run = None
        self.metadata = None

        self._load_metadata()

        self.services = self.services_config(self.metadata, thread_config)
        supported_services = vars(self.services).keys()
        self.service_list = self._build_services_list(supported_services, services, skipped_services)

    def authenticate(self):
        """
        Authenticate to the provider using provided credentials
        :return:
        """
        pass

    def preprocessing(self, ip_ranges=None, ip_ranges_name_key=None):
        """
        TODO

        :return: None
        """
        ip_ranges = [] if ip_ranges is None else ip_ranges

        # Preprocessing dictated by metadata
        self._process_metadata_callbacks()

    def postprocessing(self, current_time, ruleset):
        """
        TODO

        :param config:
        :param current_time:
        :param ruleset:
        :return: None
        """
        self._update_metadata()
        self._update_last_run(current_time, ruleset)

    def fetch(self, regions=None, skipped_regions=None, partition_name=None):
        """
        Fetch resources for each service

        :param services:
        :param skipped_services:
        :param regions:
        :param skipped_regions:
        :param partition_name:
        :return:
        """
        regions = [] if regions is None else regions
        skipped_regions = [] if skipped_regions is None else skipped_regions
        # TODO: determine partition name based on regions and warn if multiple partitions...
        self.services.fetch(self.credentials, self.service_list, regions)

        # TODO implement this properly
        """
        This is quite ugly but the legacy Scout2 expects the configurations to be dictionaries.
        Eventually this should be moved to objects/attributes, but that will require significan re-write.
        """
        report = Scout2Report(self.provider_code,
                              self.profile)
        self.services = report.jsrw.to_dict(self.services)

    def _load_metadata(self):
        """
        Load the metadata as defined in the child class metadata_path attribute

        :return: None
        """
        # Load metadata
        with open(self.metadata_path, 'rt') as f:
            self.metadata = json.load(f)

    def _build_services_list(self, supported_services, services, skipped_services):
        return [s for s in supported_services if (services == [] or s in services) and s not in skipped_services]

    def _update_last_run(self, current_time, ruleset):
        last_run = {}
        last_run['time'] = current_time.strftime("%Y-%m-%d %H:%M:%S%z")
        last_run['cmd'] = ' '.join(sys.argv)
        last_run['version'] = scout2_version
        last_run['ruleset_name'] = ruleset.name
        last_run['ruleset_about'] = ruleset.about
        last_run['summary'] = {}
        for service in self.services:
            last_run['summary'][service] = {'checked_items': 0, 'flagged_items': 0, 'max_level': 'warning',
                                            'rules_count': 0, 'resources_count': 0}
            if self.services[service] == None:
                # Not supported yet
                continue
            elif 'findings' in self.services[service]:
                for finding in self.services[service]['findings'].values():
                    last_run['summary'][service]['rules_count'] += 1
                    last_run['summary'][service]['checked_items'] += finding['checked_items']
                    last_run['summary'][service]['flagged_items'] += finding['flagged_items']
                    items = finding.get('items', [])
                    if last_run['summary'][service]['max_level'] != 'danger' and len(items) > 0:
                        last_run['summary'][service]['max_level'] = finding['level']
            # Total number of resources
            for key in self.services[service]:
                if key != 'regions_count' and key.endswith('_count'):
                    last_run['summary'][service]['resources_count'] += self.services[service][key]
        self.last_run = last_run

    def _update_metadata(self):
        service_map = {}
        for service_group in self.metadata:
            for service in self.metadata[service_group]:
                if service not in self.service_list:
                    continue
                if 'hidden' in self.metadata[service_group][service] and \
                        self.metadata[service_group][service]['hidden'] == True:
                    continue
                if 'resources' not in self.metadata[service_group][service]:
                    continue
                service_map[service] = service_group
                for resource in self.metadata[service_group][service]['resources']:
                    # full_path = path if needed
                    if not 'full_path' in self.metadata[service_group][service]['resources'][resource]:
                        self.metadata[service_group][service]['resources'][resource]['full_path'] = \
                            self.metadata[service_group][service]['resources'][resource]['path']
                    # Script is the full path minus "id" (TODO: change that)
                    if not 'script' in self.metadata[service_group][service]['resources'][resource]:
                        self.metadata[service_group][service]['resources'][resource]['script'] = '.'.join(
                            [x for x in
                             self.metadata[service_group][service]['resources'][resource]['full_path'].split(
                                 '.') if x != 'id'])
                    
                    # Update counts
                    service_config = self.services[service]
                    if not service_config :
                        continue
                    
                    count = '%s_count' % resource
                    if resource != 'regions':
                        if 'regions' in service_config.keys() and isinstance(service_config['regions'], dict):
                            self.metadata[service_group][service]['resources'][resource]['count'] = 0
                            for region in service_config['regions']:
                                if count in service_config['regions'][region].keys():
                                    self.metadata[service_group][service]['resources'][resource]['count'] += \
                                        service_config['regions'][region][count]
                        else:
                            try:
                                self.metadata[service_group][service]['resources'][resource]['count'] = \
                                    service_config[count]
                            except Exception as e:
                                printException(e)
                    else:
                        self.metadata[service_group][service]['resources'][resource]['count'] = len(service_config['regions'])


    def manage_object(self, object, attr, init, callback=None):
        """
        This is a quick-fix copy of Opine's manage_dictionary in order to support the new ScoutSuite object which isn't
        a dict
        """
        if type(object) == dict:
            if not str(attr) in object:
                object[str(attr)] = init
                self.manage_object(object, attr, init)
        else:
            if not hasattr(object, attr):
                setattr(object, attr, init)
                self.manage_object(object, attr, init)
        if callback:
            callback(getattr(object, attr))
        return object

    def _process_metadata_callbacks(self):
        """
        Iterates through each type of resource and, when callbacks have been
        configured in the config metadata, recurse through each resource and calls
        each callback.

        :param self.config:                  The entire AWS configuration object
        :return:                            None
        """
        # Service-level summaries
        for service_group in self.metadata:
            for service in self.metadata[service_group]:
                if service == 'summaries':
                    continue
                # Reset external attack surface
                if 'summaries' in self.metadata[service_group][service]:
                    for summary in self.metadata[service_group][service]['summaries']:
                        if summary == 'external attack surface' and \
                                service in self.services and \
                                'external_attack_surface' in self.services[service]:
                            self.services[service].pop('external_attack_surface')
                # Reset all global summaries
                if hasattr(self, 'service_groups'):
                    del self.service_groups
                # Resources
                for resource_type in self.metadata[service_group][service]['resources']:
                    if 'callbacks' in self.metadata[service_group][service]['resources'][resource_type]:
                        current_path = ['services', service]
                        target_path = self.metadata[service_group][service]['resources'][resource_type][
                                          'path'].replace('.id', '').split('.')[2:]
                        callbacks = self.metadata[service_group][service]['resources'][resource_type][
                            'callbacks']
                        self._new_go_to_and_do(self.services[service],
                                               target_path,
                                               current_path,
                                               callbacks)
                # Summaries
                if 'summaries' in self.metadata[service_group][service]:
                    for summary in self.metadata[service_group][service]['summaries']:
                        if 'callbacks' in self.metadata[service_group][service]['summaries'][summary]:
                            current_path = ['services', service]
                            for callback in self.metadata[service_group][service]['summaries'][summary][
                                'callbacks']:
                                callback_name = callback[0]
                                callback_args = copy.deepcopy(callback[1])
                                target_path = callback_args.pop('path').replace('.id', '').split('.')[2:]
                                callbacks = [[callback_name, callback_args]]
                                self._new_go_to_and_do(self.services[service],
                                                       target_path,
                                                       current_path,
                                                       callbacks)
        # Group-level summaries
        for service_group in self.metadata:
            if 'summaries' in self.metadata[service_group]:
                for summary in self.metadata[service_group]['summaries']:
                    current_path = ['services', service]
                    for callback in self.metadata[service_group]['summaries'][summary]['callbacks']:
                        callback_name = callback[0]
                        callback_args = copy.deepcopy(callback[1])
                        target_path = self.metadata[service_group]['summaries'][summary]['path'].split('.')
                        # quick fix as legacy Scout2 expects "self" to be a dict
                        target_object = self
                        for p in target_path:
                            self.manage_object(target_object, p, {})
                            if type(target_object) == dict:
                                target_object = target_object[p]
                            else:
                                target_object = getattr(target_object, p)
                        if callback_name == 'merge':
                            for service in self.metadata[service_group]:
                                if service == 'summaries':
                                    continue
                                if 'summaries' in self.metadata[service_group][service] and \
                                        summary in self.metadata[service_group][service]['summaries']:
                                    try:
                                        source = get_object_at(self,
                                                               self.metadata[service_group][service]['summaries'][summary]['path'].split('.'))
                                    except Exception as e:
                                        source = {}
                                    target_object.update(source)

        return None

    def _go_to_and_do(self, current_config, path, current_path, callback, callback_args=None):
        """
        Recursively go to a target and execute a callback
        """
        try:

            key = path.pop(0)
            if not current_config:
                current_config = self.config
            if not current_path:
                current_path = []
            keys = key.split('.')
            if len(keys) > 1:
                while True:
                    key = keys.pop(0)
                    if not len(keys):
                        break
                    current_path.append(key)
                    current_config = current_config[key]
            # if hasattr(current_config, key):
            if key in current_config:
                current_path.append(key)
                # current_config_key = getattr(current_config, key)
                current_config_key = current_config[key]
                for (i, value) in enumerate(list(current_config_key)):
                    if len(path) == 0:
                        if type(current_config_key == dict) and type(value) != dict and type(value) != list:
                            callback(current_config_key[value], path, current_path, value, callback_args)
                        else:
                            callback(current_config, path, current_path, value, callback_args)
                    else:
                        tmp = copy.deepcopy(current_path)
                        try:
                            tmp.append(value)
                            self._go_to_and_do(current_config_key[value], copy.deepcopy(path), tmp, callback,
                                               callback_args)
                        except:
                            tmp.pop()
                            tmp.append(i)
                            self._go_to_and_do(current_config_key[i], copy.deepcopy(path), tmp, callback,
                                               callback_args)

        except Exception as e:
            printException(e)
            printInfo('Path: %s' % str(current_path))
            printInfo('Key = %s' % str(key) if 'key' in locals() else 'not defined')
            printInfo('Value = %s' % str(value) if 'value' in locals() else 'not defined')
            printInfo('Path = %s' % path)

    def _new_go_to_and_do(self, current_config, path, current_path, callbacks):
        """
        Recursively go to a target and execute a callback
        """
        try:

            key = path.pop(0)
            if not current_config:
                current_config = self.config
            if not current_path:
                current_path = []
            keys = key.split('.')
            if len(keys) > 1:
                while True:
                    key = keys.pop(0)
                    if not len(keys):
                        break
                    current_path.append(key)
                    current_config = current_config[key]
            if key in current_config:
                current_path.append(key)
                for (i, value) in enumerate(list(current_config[key])):
                    if len(path) == 0:
                        for callback_info in callbacks:
                            callback_name = callback_info[0]

                            # callback = globals()[callback_name]
                            callback = getattr(self, callback_name)

                            callback_args = callback_info[1]
                            if type(current_config[key] == dict) and type(value) != dict and type(value) != list:
                                callback(current_config[key][value],
                                         path,
                                         current_path,
                                         value,
                                         callback_args)
                            else:
                                callback(current_config, path, current_path, value, callback_args)
                    else:
                        tmp = copy.deepcopy(current_path)
                        try:
                            tmp.append(value)
                            self._new_go_to_and_do(current_config[key][value], copy.deepcopy(path), tmp,
                                                   callbacks)
                        except:
                            tmp.pop()
                            tmp.append(i)
                            self._new_go_to_and_do(current_config[key][i], copy.deepcopy(path), tmp, callbacks)
        except Exception as e:
            printException(e)
            printInfo('Path: %s' % str(current_path))
            printInfo('Key = %s' % str(key) if 'key' in locals() else 'not defined')
            printInfo('Value = %s' % str(value) if 'value' in locals() else 'not defined')
            printInfo('Path = %s' % path)
